# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FOD2FIXEL_METADATA = Metadata(
    id="0f495be6ac07c3442550036e3ffcf4914a2efeca.boutiques",
    name="fod2fixel",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class Fod2fixelConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class Fod2fixelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fod2fixel(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    fixel_directory: OutputPathType
    """the output fixel directory"""
    afd: OutputPathType | None
    """output the total Apparent Fibre Density per fixel (integral of FOD lobe)
    """
    peak_amp: OutputPathType | None
    """output the amplitude of the FOD at the maximal peak per fixel """
    disp: OutputPathType | None
    """output a measure of dispersion per fixel as the ratio between FOD lobe
    integral and maximal peak amplitude """


def fod2fixel(
    fod: InputPathType,
    fixel_directory: str,
    afd: str | None = None,
    peak_amp: str | None = None,
    disp: str | None = None,
    fmls_integral: float | None = None,
    fmls_peak_value: float | None = None,
    fmls_no_thresholds: bool = False,
    fmls_lobe_merge_ratio: float | None = None,
    mask: InputPathType | None = None,
    maxnum: int | None = None,
    nii: bool = False,
    dirpeak: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fod2fixelConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Fod2fixelOutputs:
    """
    Perform segmentation of continuous Fibre Orientation Distributions (FODs) to
    produce discrete fixels.
    
    
    
    References:
    
    * Reference for the FOD segmentation method:
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT:
    Spherical-deconvolution informed filtering of tractograms. NeuroImage, 2013,
    67, 298-312 (Appendix 2)
    
    * Reference for Apparent Fibre Density (AFD):
    Raffelt, D.; Tournier, J.-D.; Rose, S.; Ridgway, G.R.; Henderson, R.;
    Crozier, S.; Salvado, O.; Connelly, A. Apparent Fibre Density: a novel
    measure for the analysis of diffusion-weighted magnetic resonance
    images.Neuroimage, 2012, 15;59(4), 3976-94.
    
    Author: Robert E. Smith (robert.smith@florey.edu.au)
    
    URL:
    https://mrtrix.readthedocs.io/en/latest/reference/commands/fod2fixel.html
    
    Args:
        fod: the input fod image.
        fixel_directory: the output fixel directory.
        afd: output the total Apparent Fibre Density per fixel (integral of FOD\
            lobe).
        peak_amp: output the amplitude of the FOD at the maximal peak per fixel.
        disp: output a measure of dispersion per fixel as the ratio between FOD\
            lobe integral and maximal peak amplitude.
        fmls_integral: threshold absolute numerical integral of positive FOD\
            lobes. Any lobe for which the integral is smaller than this threshold\
            will be discarded. Default: 0.
        fmls_peak_value: threshold peak amplitude of positive FOD lobes. Any\
            lobe for which the maximal peak amplitude is smaller than this\
            threshold will be discarded. Default: 0.1.
        fmls_no_thresholds: disable all FOD lobe thresholding; every lobe where\
            the FOD is positive will be retained.
        fmls_lobe_merge_ratio: Specify the ratio between a given FOD amplitude\
            sample between two lobes, and the smallest peak amplitude of the\
            adjacent lobes, above which those lobes will be merged. This is the\
            amplitude of the FOD at the 'bridge' point between the two lobes,\
            divided by the peak amplitude of the smaller of the two adjoining\
            lobes. A value of 1.0 will never merge two lobes into one; a value of\
            0.0 will always merge lobes unless they are bisected by a zero-valued\
            crossing. Default: 1.
        mask: only perform computation within the specified binary brain mask\
            image.
        maxnum: maximum number of fixels to output for any particular voxel\
            (default: no limit).
        nii: output the directions and index file in nii format (instead of the\
            default mif).
        dirpeak: define the fixel direction as that of the lobe's maximal peak\
            as opposed to its weighted mean direction (the default).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fod2fixelOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOD2FIXEL_METADATA)
    cargs = []
    cargs.append("fod2fixel")
    if afd is not None:
        cargs.extend([
            "-afd",
            afd
        ])
    if peak_amp is not None:
        cargs.extend([
            "-peak_amp",
            peak_amp
        ])
    if disp is not None:
        cargs.extend([
            "-disp",
            disp
        ])
    if fmls_integral is not None:
        cargs.extend([
            "-fmls_integral",
            str(fmls_integral)
        ])
    if fmls_peak_value is not None:
        cargs.extend([
            "-fmls_peak_value",
            str(fmls_peak_value)
        ])
    if fmls_no_thresholds:
        cargs.append("-fmls_no_thresholds")
    if fmls_lobe_merge_ratio is not None:
        cargs.extend([
            "-fmls_lobe_merge_ratio",
            str(fmls_lobe_merge_ratio)
        ])
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if maxnum is not None:
        cargs.extend([
            "-maxnum",
            str(maxnum)
        ])
    if nii:
        cargs.append("-nii")
    if dirpeak:
        cargs.append("-dirpeak")
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(fod))
    cargs.append(fixel_directory)
    ret = Fod2fixelOutputs(
        root=execution.output_file("."),
        fixel_directory=execution.output_file(fixel_directory),
        afd=execution.output_file(afd) if (afd is not None) else None,
        peak_amp=execution.output_file(peak_amp) if (peak_amp is not None) else None,
        disp=execution.output_file(disp) if (disp is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FOD2FIXEL_METADATA",
    "Fod2fixelConfig",
    "Fod2fixelOutputs",
    "fod2fixel",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DSVM_METADATA = Metadata(
    id="0f6c752a6ae915fa2d5ee354357ef3df203df0d6.boutiques",
    name="3dsvm",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dsvmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3dsvm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_model: OutputPathType
    """Output model file in .1D format."""
    out_alpha: OutputPathType
    """Output alphas file in .1D format."""
    out_bucket: OutputPathType
    """Output bucket file in .1D format."""
    out_predictions: OutputPathType
    """Output predictions file in .1D format."""


def v_3dsvm(
    runner: Runner | None = None,
) -> V3dsvmOutputs:
    """
    Support vector machine analysis of brain data using the SVM-light package.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dsvm.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dsvmOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DSVM_METADATA)
    cargs = []
    cargs.append("3dsvm")
    cargs.append("[OPTIONS]")
    ret = V3dsvmOutputs(
        root=execution.output_file("."),
        out_model=execution.output_file("model_{output}.1D"),
        out_alpha=execution.output_file("alpha_{output}.1D"),
        out_bucket=execution.output_file("bucket_{output}.1D"),
        out_predictions=execution.output_file("predictions_{output}.1D"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dsvmOutputs",
    "V_3DSVM_METADATA",
    "v_3dsvm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

IMGLOB_METADATA = Metadata(
    id="04388dd1ed84da67a1aac9da7d9b7d85a0f8a49c.boutiques",
    name="imglob",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class ImglobOutputs(typing.NamedTuple):
    """
    Output object returned when calling `imglob(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def imglob(
    input_list: list[str],
    multiple_extensions: bool = False,
    runner: Runner | None = None,
) -> ImglobOutputs:
    """
    Tool for generating image lists with specific extensions.
    
    Author: Software Author
    
    Args:
        input_list: List of image names or file paths.
        multiple_extensions: Output list of images with full extensions.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImglobOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMGLOB_METADATA)
    cargs = []
    cargs.append("imglob")
    if multiple_extensions:
        cargs.append("-extensions")
    cargs.extend(input_list)
    ret = ImglobOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "IMGLOB_METADATA",
    "ImglobOutputs",
    "imglob",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_DIFF_METADATA = Metadata(
    id="9e9ee8f4e790b360b9a145fdfd02de7adec621c8.boutiques",
    name="3dDiff",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dDiffOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_diff(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_log: OutputPathType
    """Log file containing the element-wise differences."""


def v_3d_diff(
    dataset_a: InputPathType,
    dataset_b: InputPathType,
    runner: Runner | None = None,
) -> V3dDiffOutputs:
    """
    A program to examine element-wise differences between two images.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dDiff.html
    
    Args:
        dataset_a: First input dataset for comparison.
        dataset_b: Second input dataset for comparison.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dDiffOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_DIFF_METADATA)
    cargs = []
    cargs.append("3dDiff")
    cargs.append("[display_opt]")
    cargs.append("[-tol")
    cargs.append("TOLERANCE]")
    cargs.append("[-mask")
    cargs.append("MASK]")
    cargs.append("-a")
    cargs.extend([
        "-a",
        execution.input_file(dataset_a)
    ])
    cargs.append("-b")
    cargs.extend([
        "-b",
        execution.input_file(dataset_b)
    ])
    ret = V3dDiffOutputs(
        root=execution.output_file("."),
        output_log=execution.output_file(pathlib.Path(dataset_a).name + "_vs_" + pathlib.Path(dataset_b).name + ".log"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dDiffOutputs",
    "V_3D_DIFF_METADATA",
    "v_3d_diff",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__DRIVE_SUMA_METADATA = Metadata(
    id="f0a765d1c6ee3fa4cf10d62c557d251571658b10.boutiques",
    name="@DriveSuma",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VDriveSumaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__drive_suma(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__drive_suma(
    h_aspx: bool = False,
    runner: Runner | None = None,
) -> VDriveSumaOutputs:
    """
    A script to demonstrate how to drive suma from the command line.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/@DriveSuma.html
    
    Args:
        h_aspx: Sphinx formatted help string with auto tagging of options.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VDriveSumaOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__DRIVE_SUMA_METADATA)
    cargs = []
    cargs.append("@DriveSuma")
    if h_aspx:
        cargs.append("-h_aspx")
    ret = VDriveSumaOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VDriveSumaOutputs",
    "V__DRIVE_SUMA_METADATA",
    "v__drive_suma",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BIANCA_PERIVENT_DEEP_METADATA = Metadata(
    id="b4075edf8e76b937e80559712e3fd8686875ca35.boutiques",
    name="bianca_perivent_deep",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class BiancaPeriventDeepOutputs(typing.NamedTuple):
    """
    Output object returned when calling `bianca_perivent_deep(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    perivent_map: OutputPathType
    """Binary image of periventricular WMH"""
    deepwm_map: OutputPathType
    """Binary image of deep WMH"""
    wmh_volumes: OutputPathType
    """Text file with volumes of total, periventricular, and deep WMHs (if
    DO_STATS is 2)"""


def bianca_perivent_deep(
    thresholded_binarised_wmh_map: InputPathType,
    ventricles_mask: InputPathType,
    minclustersize: float,
    do_stats: int,
    outputdir: str,
    runner: Runner | None = None,
) -> BiancaPeriventDeepOutputs:
    """
    Separates WMH into periventricular and deep WMH, saves two separate binary
    images, and calculates volume of total and separate WMHs.
    
    Author: BIANCA Development Team
    
    Args:
        thresholded_binarised_wmh_map: Thresholded and binarized WMH map\
            (output of BIANCA thresholded and binarized).
        ventricles_mask: Ventricles mask you created using the exclusion mask\
            script. If T1 and FLAIR were not in the same space, ventricle mask\
            needs to be registered to FLAIR (and binarized).
        minclustersize: Minimum cluster (volume) size in voxels to consider.\
            Use 0 for no cluster thresholding.
        do_stats: Option to calculate volumes: 0 to only produce images without\
            volume calculation, 1 to calculate volumes for total, periventricular\
            and deep WMH and display on the screen, 2 to calculate volumes and save\
            them in a file.
        outputdir: Directory to save the output files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BiancaPeriventDeepOutputs`).
    """
    if not (0 <= do_stats <= 2): 
        raise ValueError(f"'do_stats' must be between 0 <= x <= 2 but was {do_stats}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(BIANCA_PERIVENT_DEEP_METADATA)
    cargs = []
    cargs.append("bianca_perivent_deep")
    cargs.append(execution.input_file(thresholded_binarised_wmh_map))
    cargs.append(execution.input_file(ventricles_mask))
    cargs.append(str(minclustersize))
    cargs.append(str(do_stats))
    cargs.append(outputdir)
    ret = BiancaPeriventDeepOutputs(
        root=execution.output_file("."),
        perivent_map=execution.output_file(outputdir + "/perivent_map.nii.gz"),
        deepwm_map=execution.output_file(outputdir + "/deepwm_map.nii.gz"),
        wmh_volumes=execution.output_file(outputdir + "/WMH_tot_pvent_deep_10mm.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BIANCA_PERIVENT_DEEP_METADATA",
    "BiancaPeriventDeepOutputs",
    "bianca_perivent_deep",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MAKE_RANDOM_TIMING_PY_METADATA = Metadata(
    id="b4d28915fbea90096e2020dbc606016af5151ee2.boutiques",
    name="make_random_timing.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class MakeRandomTimingPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `make_random_timing_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    stim_output: OutputPathType
    """Stimulus timing output file"""


def make_random_timing_py(
    runner: Runner | None = None,
) -> MakeRandomTimingPyOutputs:
    """
    Create random stimulus timing files for use with AFNI 3dDeconvolve.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/make_random_timing.py.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MakeRandomTimingPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MAKE_RANDOM_TIMING_PY_METADATA)
    cargs = []
    cargs.append("make_random_timing.py")
    cargs.append("[INPUT_PARAMS]")
    ret = MakeRandomTimingPyOutputs(
        root=execution.output_file("."),
        stim_output=execution.output_file("[PREFIX]_*.1D"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MAKE_RANDOM_TIMING_PY_METADATA",
    "MakeRandomTimingPyOutputs",
    "make_random_timing_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FABBER_METADATA = Metadata(
    id="7b30d160c8a5a0d761389119c4ba88f3aec8bd00.boutiques",
    name="fabber",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class FabberOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fabber(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    paramnames_file: OutputPathType
    """File containing the names of the model parameters"""
    model_fit_file: OutputPathType
    """The model fit output file"""
    residuals_file: OutputPathType
    """The residuals output file"""
    model_extras_file: OutputPathType
    """The model extras output file"""
    mvn_file: OutputPathType
    """The MVN distributions output file"""
    mean_file: OutputPathType
    """The parameter means output file"""
    std_file: OutputPathType
    """The parameter standard deviations output file"""
    var_file: OutputPathType
    """The parameter variances output file"""
    zstat_file: OutputPathType
    """The parameter Zstats output file"""
    noise_mean_file: OutputPathType
    """The noise means output file"""
    noise_std_file: OutputPathType
    """The noise standard deviations output file"""
    free_energy_file: OutputPathType
    """The free energy output file"""


def fabber(
    runner: Runner | None = None,
) -> FabberOutputs:
    """
    Fabber is a tool for model-based Bayesian analysis of time-series data.
    
    Author: FSL Community
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/Fabber
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FabberOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FABBER_METADATA)
    cargs = []
    cargs.append("fabber")
    cargs.append("[OPTIONS]")
    ret = FabberOutputs(
        root=execution.output_file("."),
        paramnames_file=execution.output_file("[OUTPUT_DIRECTORY]/paramnames.txt"),
        model_fit_file=execution.output_file("[OUTPUT_DIRECTORY]/model_fit.nii.gz"),
        residuals_file=execution.output_file("[OUTPUT_DIRECTORY]/residuals.nii.gz"),
        model_extras_file=execution.output_file("[OUTPUT_DIRECTORY]/model_extras.nii.gz"),
        mvn_file=execution.output_file("[OUTPUT_DIRECTORY]/mvn.nii.gz"),
        mean_file=execution.output_file("[OUTPUT_DIRECTORY]/mean.nii.gz"),
        std_file=execution.output_file("[OUTPUT_DIRECTORY]/std.nii.gz"),
        var_file=execution.output_file("[OUTPUT_DIRECTORY]/var.nii.gz"),
        zstat_file=execution.output_file("[OUTPUT_DIRECTORY]/zstat.nii.gz"),
        noise_mean_file=execution.output_file("[OUTPUT_DIRECTORY]/noise_mean.nii.gz"),
        noise_std_file=execution.output_file("[OUTPUT_DIRECTORY]/noise_std.nii.gz"),
        free_energy_file=execution.output_file("[OUTPUT_DIRECTORY]/free_energy.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FABBER_METADATA",
    "FabberOutputs",
    "fabber",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_ZIPPER_ZAPPER_METADATA = Metadata(
    id="042789412d0abd60c98ba23fababe48904ad9cff.boutiques",
    name="3dZipperZapper",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dZipperZapperOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_zipper_zapper(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    bad_slice_mask: OutputPathType
    """Mask of potentially bad slices across the input dataset."""
    bad_volumes_list: OutputPathType
    """1D file containing a list of the bad volumes."""
    per_volume_params: OutputPathType
    """1D file of the per-volume parameters used to detect badness."""
    calculated_slices: OutputPathType
    """1D file of the slices within which calculations were made."""
    good_volumes_selector: OutputPathType
    """Text file with the selector string of *good* volumes."""


def v_3d_zipper_zapper(
    input_file: InputPathType,
    output_prefix: str,
    mask_file: InputPathType | None = None,
    min_slice_nvox: float | None = None,
    min_streak_len: float | None = None,
    do_out_slice_param: bool = False,
    no_out_bad_mask: bool = False,
    no_out_text_vals: bool = False,
    dont_use_streak: bool = False,
    dont_use_drop: bool = False,
    dont_use_corr: bool = False,
    min_streak_val: float | None = None,
    min_drop_frac: float | None = None,
    min_drop_diff: float | None = None,
    min_corr_len: float | None = None,
    min_corr_corr: float | None = None,
    runner: Runner | None = None,
) -> V3dZipperZapperOutputs:
    """
    A basic program to highlight problematic volumes in data sets, especially
    EPI/DWI data sets with interleaved acquisition.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dZipperZapper.html
    
    Args:
        input_file: Input 3D+time file of DWIs or EPIs.
        output_prefix: Prefix for output file name.
        mask_file: Optional input of a single volume mask file, which gets\
            applied to each volume in the input file.
        min_slice_nvox: Set the minimum number of voxels to be in the mask for\
            a given slice to be included in the calculations.
        min_streak_len: Minimum number of slices in a row to look for\
            fluctuations within.
        do_out_slice_param: Output the map of slice parameters.
        no_out_bad_mask: Do not output the mask of 'bad' slices.
        no_out_text_vals: Do not output the 1D files of the slice parameter\
            values.
        dont_use_streak: Turn off the 'streak' criterion for identifying bad\
            slices.
        dont_use_drop: Turn off the 'drop' criterion for identifying bad\
            slices.
        dont_use_corr: Turn off the 'corr' criterion for identifying bad\
            slices.
        min_streak_val: Minimum magnitude of voxelwise relative differences to\
            perhaps be problematic.
        min_drop_frac: Minimum fraction for judging if the change in 'slice\
            parameter' differences between neighboring slices might be a sign of\
            badness.
        min_drop_diff: Minimum 'slice parameter' value within a single slice\
            that might be considered a bad sign.
        min_corr_len: Minimum number of slices in a row to look for consecutive\
            anticorrelations in brightness differences.
        min_corr_corr: Threshold for the magnitude of anticorrelations to be\
            considered potentially bad.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dZipperZapperOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_ZIPPER_ZAPPER_METADATA)
    cargs = []
    cargs.append("3dZipperZapper")
    cargs.append("-input")
    cargs.append(execution.input_file(input_file))
    cargs.append("-prefix")
    cargs.append(output_prefix)
    if mask_file is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask_file)
        ])
    if min_slice_nvox is not None:
        cargs.extend([
            "-min_slice_nvox",
            str(min_slice_nvox)
        ])
    if min_streak_len is not None:
        cargs.extend([
            "-min_streak_len",
            str(min_streak_len)
        ])
    if do_out_slice_param:
        cargs.append("-do_out_slice_param")
    if no_out_bad_mask:
        cargs.append("-no_out_bad_mask")
    if no_out_text_vals:
        cargs.append("-no_out_text_vals")
    if dont_use_streak:
        cargs.append("-dont_use_streak")
    if dont_use_drop:
        cargs.append("-dont_use_drop")
    if dont_use_corr:
        cargs.append("-dont_use_corr")
    if min_streak_val is not None:
        cargs.extend([
            "-min_streak_val",
            str(min_streak_val)
        ])
    if min_drop_frac is not None:
        cargs.extend([
            "-min_drop_frac",
            str(min_drop_frac)
        ])
    if min_drop_diff is not None:
        cargs.extend([
            "-min_drop_diff",
            str(min_drop_diff)
        ])
    if min_corr_len is not None:
        cargs.extend([
            "-min_corr_len",
            str(min_corr_len)
        ])
    if min_corr_corr is not None:
        cargs.extend([
            "-min_corr_corr",
            str(min_corr_corr)
        ])
    ret = V3dZipperZapperOutputs(
        root=execution.output_file("."),
        bad_slice_mask=execution.output_file(output_prefix + "_badmask.nii.gz"),
        bad_volumes_list=execution.output_file(output_prefix + "_badvols.1D"),
        per_volume_params=execution.output_file(output_prefix + "_param.1D"),
        calculated_slices=execution.output_file(output_prefix + "_sli.1D"),
        good_volumes_selector=execution.output_file(output_prefix + "_goodvols.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dZipperZapperOutputs",
    "V_3D_ZIPPER_ZAPPER_METADATA",
    "v_3d_zipper_zapper",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TBSS_2_REG_METADATA = Metadata(
    id="bc70ea875b4f3176bc68c67f7dd2ed5045d3e370.boutiques",
    name="tbss_2_reg",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class Tbss2RegOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tbss_2_reg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def tbss_2_reg(
    runner: Runner | None = None,
) -> Tbss2RegOutputs:
    """
    TBSS utility for target selection and registration for Tract-Based Spatial
    Statistics (TBSS) analysis.
    
    Author: Oxford Centre for Functional MRI of the Brain (FMRIB)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/TBSS/UserGuide
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tbss2RegOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TBSS_2_REG_METADATA)
    cargs = []
    cargs.append("tbss_2_reg")
    cargs.append("[TARGET_SELECTION_OPTION]")
    ret = Tbss2RegOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TBSS_2_REG_METADATA",
    "Tbss2RegOutputs",
    "tbss_2_reg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANTS_REGISTRATION_SYN_SH_METADATA = Metadata(
    id="5b04a385a189dd578b48a9b81f177c398fc3e1fd.boutiques",
    name="ants_registration_syn_sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class AntsRegistrationSynShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_registration_syn_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transform: OutputPathType
    """The output transform files."""


def ants_registration_syn_sh(
    image_dimension: typing.Literal[2, 3],
    fixed_image: InputPathType,
    moving_image: InputPathType,
    output_prefix: str,
    threads: int | None = None,
    initial_transform: list[str] | None = None,
    transform_type: typing.Literal["t", "r", "a", "s", "sr", "so", "b", "br", "bo"] | None = None,
    radius: int | None = None,
    spline_distance: int | None = None,
    gradient_step: float | None = None,
    masks: str | None = None,
    precision_type: typing.Literal["f", "d"] | None = None,
    use_histogram_matching: typing.Literal[0, 1] | None = None,
    use_repro_mode: typing.Literal[0, 1] | None = None,
    collapse_output_transforms: typing.Literal[0, 1] | None = None,
    random_seed: int | None = None,
    runner: Runner | None = None,
) -> AntsRegistrationSynShOutputs:
    """
    This script performs image registration using the SyN transform from the ANTs
    suite. The user can specify the dimensionality of the images, the fixed and
    moving images, and the output prefix. It supports various types of
    transformations and optional parameters such as initial transforms, gradient
    step size, mask usage, and more.
    
    Author: Brian B. Avants, Nick Tustison, and Gang Song
    
    URL: https://github.com/ANTsX/ANTs/
    
    Args:
        image_dimension: Image dimension: 2 or 3 (for 2 or 3-dimensional\
            registration of a single volume).
        fixed_image: Fixed image(s) or source image(s) or reference image(s).
        moving_image: Moving image(s) or target image(s).
        output_prefix: A prefix that is prepended to all output files.
        threads: Number of threads (default =\
            ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS if defined, otherwise 1).
        initial_transform: Initial transform(s) --- order specified on the\
            command line matters.
        transform_type: Transform type (default = 's'). Options:\
            - t: translation (1 stage)\
            - r: rigid (1 stage)\
            - a: rigid + affine (2 stages)\
            - s: rigid + affine + deformable syn (3 stages)\
            - sr: rigid + deformable syn (2 stages)\
            - so: deformable syn only (1 stage)\
            - b: rigid + affine + deformable b-spline syn (3 stages)\
            - br: rigid + deformable b-spline syn (2 stages)\
            - bo: deformable b-spline syn only (1 stage).
        radius: Radius for cross correlation metric used during SyN stage\
            (default = 4).
        spline_distance: Spline distance for deformable B-spline SyN transform\
            (default = 26).
        gradient_step: Gradient step size for SyN and B-spline SyN (default =\
            0.1).
        masks: Mask(s) for the fixed image space, or for the fixed and moving\
            image space in the format 'fixedMask,MovingMask'. Use -x once to\
            specify mask(s) to be used for all stages or use -x for each 'stage'\
            (cf -t option). If no mask is to be used for a particular stage, the\
            keyword 'NULL' should be used in place of file names.
        precision_type: Precision type (default = 'd'). Options:\
            - f: float\
            - d: double.
        use_histogram_matching: Use histogram matching (default = 0). Options:\
            - 0: false\
            - 1: true.
        use_repro_mode: Use 'repro' mode for exact reproducibility of output.\
            Uses GC metric for linear stages and a fixed random seed (default = 0).\
            Options:\
            - 0: false\
            - 1: true.
        collapse_output_transforms: Collapse output transforms (default = 1).\
            Options:\
            - 0: false\
            - 1: true.
        random_seed: Fix random seed to an int value.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsRegistrationSynShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_REGISTRATION_SYN_SH_METADATA)
    cargs = []
    cargs.append("antsRegistrationSyN.sh")
    cargs.extend([
        "-d",
        str(image_dimension)
    ])
    cargs.extend([
        "-f",
        execution.input_file(fixed_image)
    ])
    cargs.extend([
        "-m",
        execution.input_file(moving_image)
    ])
    cargs.extend([
        "-o",
        output_prefix
    ])
    if threads is not None:
        cargs.extend([
            "-n",
            str(threads)
        ])
    if initial_transform is not None:
        cargs.extend([
            "-i",
            *initial_transform
        ])
    if transform_type is not None:
        cargs.extend([
            "-t",
            transform_type
        ])
    if radius is not None:
        cargs.extend([
            "-r",
            str(radius)
        ])
    if spline_distance is not None:
        cargs.extend([
            "-s",
            str(spline_distance)
        ])
    if gradient_step is not None:
        cargs.extend([
            "-g",
            str(gradient_step)
        ])
    if masks is not None:
        cargs.extend([
            "-x",
            masks
        ])
    if precision_type is not None:
        cargs.extend([
            "-p",
            precision_type
        ])
    if use_histogram_matching is not None:
        cargs.extend([
            "-j",
            str(use_histogram_matching)
        ])
    if use_repro_mode is not None:
        cargs.extend([
            "-y",
            str(use_repro_mode)
        ])
    if collapse_output_transforms is not None:
        cargs.extend([
            "-z",
            str(collapse_output_transforms)
        ])
    if random_seed is not None:
        cargs.extend([
            "-e",
            str(random_seed)
        ])
    ret = AntsRegistrationSynShOutputs(
        root=execution.output_file("."),
        output_transform=execution.output_file(output_prefix + "*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANTS_REGISTRATION_SYN_SH_METADATA",
    "AntsRegistrationSynShOutputs",
    "ants_registration_syn_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__RETINO_PROC_METADATA = Metadata(
    id="68decbff812b0a172bd91c6f856d4c4c196ae1d0.boutiques",
    name="@RetinoProc",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VRetinoProcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__retino_proc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__retino_proc(
    runner: Runner | None = None,
) -> VRetinoProcOutputs:
    """
    A script to process retinotopic FMRI data, using AFNI's 3dRetinoPhase and
    SurfRetinMap.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/@RetinoProc.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VRetinoProcOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__RETINO_PROC_METADATA)
    cargs = []
    cargs.append("@RetinoProc")
    cargs.append("[INPUT_FILES]")
    cargs.append("[OPTIONS]")
    ret = VRetinoProcOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VRetinoProcOutputs",
    "V__RETINO_PROC_METADATA",
    "v__retino_proc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_1D_TOOL_PY_METADATA = Metadata(
    id="29ac72eff5de4e478010309bcc7d43ce29550d2f.boutiques",
    name="1d_tool.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V1dToolPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_1d_tool_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType
    """Resulting 1D file"""


def v_1d_tool_py(
    runner: Runner | None = None,
) -> V1dToolPyOutputs:
    """
    A tool for manipulating and evaluating 1D files.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/1d_tool.py.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V1dToolPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_1D_TOOL_PY_METADATA)
    cargs = []
    cargs.append("1d_tool.py")
    cargs.append("[OPTIONS]")
    ret = V1dToolPyOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file("[OUTPUT_FILE]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V1dToolPyOutputs",
    "V_1D_TOOL_PY_METADATA",
    "v_1d_tool_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_FRIEDMAN_METADATA = Metadata(
    id="50b5a51e373a17a7d68d2d9d197174d06b3db11b.boutiques",
    name="3dFriedman",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dFriedmanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_friedman(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Friedman statistics output file"""


def v_3d_friedman(
    levels: int,
    datasets: list[InputPathType],
    output_prefix: str,
    workmem: int | None = None,
    voxel_num: int | None = None,
    runner: Runner | None = None,
) -> V3dFriedmanOutputs:
    """
    Performs nonparametric Friedman test for randomized complete block design
    experiments.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dFriedman.html
    
    Args:
        levels: Number of treatments.
        datasets: Data sets for each treatment.
        output_prefix: Prefix for the output files.
        workmem: Number of megabytes of RAM to use for statistical workspace.
        voxel_num: Screen output for a specific voxel number.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dFriedmanOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_FRIEDMAN_METADATA)
    cargs = []
    cargs.append("3dFriedman")
    cargs.append(str(levels))
    cargs.extend([
        "-dset",
        *[execution.input_file(f) for f in datasets]
    ])
    if workmem is not None:
        cargs.extend([
            "-workmem",
            str(workmem)
        ])
    if voxel_num is not None:
        cargs.extend([
            "-voxel",
            str(voxel_num)
        ])
    cargs.extend([
        "-out",
        output_prefix
    ])
    ret = V3dFriedmanOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_prefix + "*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dFriedmanOutputs",
    "V_3D_FRIEDMAN_METADATA",
    "v_3d_friedman",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FFTEST_METADATA = Metadata(
    id="ada650401138e86bba7251c62fa921204c0a825d.boutiques",
    name="fftest",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class FftestOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fftest(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def fftest(
    length: float,
    num_tests: float,
    vector_size: float,
    runner: Runner | None = None,
) -> FftestOutputs:
    """
    A command line tool for testing purposes.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/fftest.html
    
    Args:
        length: Length of the test.
        num_tests: Number of tests to run.
        vector_size: Vector size for the test.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FftestOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FFTEST_METADATA)
    cargs = []
    cargs.append("fftest")
    cargs.append("[-q]")
    cargs.append(str(length))
    cargs.append(str(num_tests))
    cargs.append(str(vector_size))
    ret = FftestOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FFTEST_METADATA",
    "FftestOutputs",
    "fftest",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CONVEX_HULL_METADATA = Metadata(
    id="e2f6d3358ed3ed3eeaa4c4f04b358f8c691f43a3.boutiques",
    name="ConvexHull",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ConvexHullOutputs(typing.NamedTuple):
    """
    Output object returned when calling `convex_hull(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_surf: OutputPathType
    """Output surface file"""


def convex_hull(
    runner: Runner | None = None,
) -> ConvexHullOutputs:
    """
    A program to find the convex hull, or perform a Delaunay triangulation of a set
    of points.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/ConvexHull.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvexHullOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVEX_HULL_METADATA)
    cargs = []
    cargs.append("<main_flags>")
    cargs.append("<volume_flags>")
    cargs.append("<surface_flags>")
    cargs.append("<triangulation_flags>")
    cargs.append("<output_flags>")
    cargs.append("<debug_flags>")
    cargs.append("<global_flags>")
    ret = ConvexHullOutputs(
        root=execution.output_file("."),
        out_surf=execution.output_file("[O_PREFIX]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CONVEX_HULL_METADATA",
    "ConvexHullOutputs",
    "convex_hull",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__ATLASIZE_METADATA = Metadata(
    id="92a5f64f5c4f1d01138a6ae2a38488993fd99734.boutiques",
    name="@Atlasize",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VAtlasizeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__atlasize(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    niml_file: OutputPathType
    """Generated NIML file for the atlas"""


def v__atlasize(
    runner: Runner | None = None,
) -> VAtlasizeOutputs:
    """
    Script to turn a volumetric dataset into an AFNI atlas.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/@Atlasize.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VAtlasizeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__ATLASIZE_METADATA)
    cargs = []
    cargs.append("@Atlasize")
    cargs.append("[-dset")
    cargs.append("DSET]")
    cargs.append("[-space")
    cargs.append("SPACE]")
    cargs.append("[-lab_file")
    cargs.append("FILE")
    cargs.append("cLAB")
    cargs.append("cVAL]")
    cargs.append("[-lab_file_delim")
    cargs.append("COL_DELIM]")
    cargs.append("[-longnames")
    cargs.append("cLONGNAME]")
    cargs.append("[-last_longname_col")
    cargs.append("cLASTLONGNAME]")
    cargs.append("[-atlas_type")
    cargs.append("TP]")
    cargs.append("[-atlas_description")
    cargs.append("DESCRP]")
    cargs.append("[-atlas_name")
    cargs.append("NAME]")
    cargs.append("[-auto_backup]")
    cargs.append("[-centers]")
    cargs.append("[-centertype")
    cargs.append("TYPE]")
    cargs.append("[-centermask")
    cargs.append("DSET]")
    cargs.append("[-skip_novoxels]")
    cargs.append("[-h_web]")
    cargs.append("[-hweb]")
    cargs.append("[-h_view]")
    cargs.append("[-hview]")
    cargs.append("[-all_opts]")
    cargs.append("[-h_find")
    cargs.append("WORD]")
    ret = VAtlasizeOutputs(
        root=execution.output_file("."),
        niml_file=execution.output_file("[DSET].niml"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VAtlasizeOutputs",
    "V__ATLASIZE_METADATA",
    "v__atlasize",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_REORDER_METADATA = Metadata(
    id="99290b5497c22077215e5388634353b6a67904bc.boutiques",
    name="cifti-reorder",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class CiftiReorderOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_reorder(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the reordered cifti file"""


def cifti_reorder(
    cifti_in: InputPathType,
    direction: str,
    reorder_list: str,
    cifti_out: str,
    runner: Runner | None = None,
) -> CiftiReorderOutputs:
    """
    Reorder the parcels or scalar/label maps in a cifti file.
    
    The mapping along the specified direction must be parcels, scalars, or
    labels. For pscalar or ptseries, use COLUMN to reorder the parcels. For
    dlabel, use ROW. The <reorder-list> file must contain 1-based indices
    separated by whitespace (spaces, newlines, tabs, etc), with as many indices
    as <cifti-in> has along the specified dimension. These indices specify which
    current index should end up in that position, for instance, if the current
    order is 'A B C D', and the desired order is 'D A B C', the text file should
    contain '4 1 2 3'.
    
    Author: Washington University School of Medicin
    
    Args:
        cifti_in: input cifti file.
        direction: which dimension to reorder along, ROW or COLUMN.
        reorder_list: a text file containing the desired order transformation.
        cifti_out: the reordered cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiReorderOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_REORDER_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-reorder")
    cargs.append(execution.input_file(cifti_in))
    cargs.append(direction)
    cargs.append(reorder_list)
    cargs.append(cifti_out)
    ret = CiftiReorderOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_REORDER_METADATA",
    "CiftiReorderOutputs",
    "cifti_reorder",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FSL_BOXPLOT_METADATA = Metadata(
    id="4746059f6bffa2c7da480b65c8879ee373bfee04.boutiques",
    name="fsl_boxplot",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class FslBoxplotOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fsl_boxplot(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_png: OutputPathType
    """The output boxplot image in PNG format"""


def fsl_boxplot(
    input_files: list[InputPathType],
    output_image: str,
    title: str | None = None,
    legend_file: InputPathType | None = None,
    x_label: str | None = None,
    y_label: str | None = None,
    plot_height: float | None = 450,
    plot_width: float | None = None,
    runner: Runner | None = None,
) -> FslBoxplotOutputs:
    """
    Tool for creating boxplot images from ASCII text matrices.
    
    Author: University of Oxford (Christian F. Beckmann)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        input_files: Comma-separated list of input file names (ASCII text\
            matrices, one column per boxplot).
        output_image: Output filename for the PNG file.
        title: Plot title.
        legend_file: File name of ASCII text file, one row per legend entry.
        x_label: X-axis label.
        y_label: Y-axis label.
        plot_height: Plot height in pixels (default 450).
        plot_width: Plot width in pixels (default 80*#boxplots).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FslBoxplotOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FSL_BOXPLOT_METADATA)
    cargs = []
    cargs.append("fsl_boxplot")
    cargs.append("--in")
    cargs.extend([
        "--in",
        *[execution.input_file(f) for f in input_files]
    ])
    cargs.append("--out")
    cargs.extend([
        "--out",
        output_image
    ])
    if title is not None:
        cargs.extend([
            "--title",
            title
        ])
    if legend_file is not None:
        cargs.extend([
            "--legend",
            execution.input_file(legend_file)
        ])
    if x_label is not None:
        cargs.extend([
            "--xlabel",
            x_label
        ])
    if y_label is not None:
        cargs.extend([
            "--ylabel",
            y_label
        ])
    if plot_height is not None:
        cargs.extend([
            "--height",
            str(plot_height)
        ])
    if plot_width is not None:
        cargs.extend([
            "--width",
            str(plot_width)
        ])
    ret = FslBoxplotOutputs(
        root=execution.output_file("."),
        output_png=execution.output_file(output_image + ".png"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FSL_BOXPLOT_METADATA",
    "FslBoxplotOutputs",
    "fsl_boxplot",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_GETROW_METADATA = Metadata(
    id="d8d7567b5d985de78a1fd73e1344abdc1903d6f4.boutiques",
    name="3dGetrow",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dGetrowOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_getrow(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_file: OutputPathType | None
    """Output .1D ASCII file"""


def v_3d_getrow(
    input_file: InputPathType | None = None,
    output_file: str | None = None,
    runner: Runner | None = None,
) -> V3dGetrowOutputs:
    """
    Program to extract 1 row from a dataset and write it as a .1D file.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dGetrow.html
    
    Args:
        input_file: Read input from dataset 'ddd' (instead of putting dataset\
            name at end of command line).
        output_file: Filename for output .1D ASCII file will be 'ff' (if 'ff'\
            is '-', then output is to stdout, the default).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dGetrowOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_GETROW_METADATA)
    cargs = []
    cargs.append("3dGetrow")
    cargs.append("[DIRECTION_FLAG]")
    if input_file is not None:
        cargs.extend([
            "-input",
            execution.input_file(input_file)
        ])
    if output_file is not None:
        cargs.extend([
            "-output",
            output_file
        ])
    ret = V3dGetrowOutputs(
        root=execution.output_file("."),
        out_file=execution.output_file(output_file + ".1D") if (output_file is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dGetrowOutputs",
    "V_3D_GETROW_METADATA",
    "v_3d_getrow",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRAVERAGEHEADER_METADATA = Metadata(
    id="b801a707f3ed2c058aecf4b7046515d22da3c81e.boutiques",
    name="mraverageheader",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class MraverageheaderConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class MraverageheaderOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mraverageheader(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image"""


def mraverageheader(
    input_: list[InputPathType],
    output: str,
    padding: float | None = None,
    resolution: str | None = None,
    fill: bool = False,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MraverageheaderConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MraverageheaderOutputs:
    """
    Calculate the average (unbiased) coordinate space of all input images.
    
    
    
    References:
    
    .
    
    Author: Maximilian Pietsch (maximilian.pietsch@kcl.ac.uk)
    
    URL:
    https://mrtrix.readthedocs.io/en/latest/reference/commands/mraverageheader.html
    
    Args:
        input_: the input image(s).
        output: the output image.
        padding: boundary box padding in voxels. Default: 0.
        resolution: subsampling of template compared to smallest voxel size in\
            any input image. Valid options are 'mean': unbiased but loss of\
            resolution for individual images possible, and 'max': smallest voxel\
            size of any input image defines the resolution. Default: mean.
        fill: set the intensity in the first volume of the average space to 1.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MraverageheaderOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRAVERAGEHEADER_METADATA)
    cargs = []
    cargs.append("mraverageheader")
    if padding is not None:
        cargs.extend([
            "-padding",
            str(padding)
        ])
    if resolution is not None:
        cargs.extend([
            "-resolution",
            resolution
        ])
    if fill:
        cargs.append("-fill")
    if datatype is not None:
        cargs.extend([
            "-datatype",
            datatype
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.extend([execution.input_file(f) for f in input_])
    cargs.append(output)
    ret = MraverageheaderOutputs(
        root=execution.output_file("."),
        output=execution.output_file(output),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRAVERAGEHEADER_METADATA",
    "MraverageheaderConfig",
    "MraverageheaderOutputs",
    "mraverageheader",
]



"""
FSL

FSL is a comprehensive library of analysis tools for FMRI, MRI and diffusion
brain imaging data. It runs on macOS (Intel and M1/M2/M3), Linux, and Windows
via the Windows Subsystem for Linux, and is very easy to install. Most of the
tools can be run both from the command line and as GUIs ("point-and-click"
graphical user interfaces). To quote the relevant references for FSL tools you
should look in the individual tools' manual pages, and also please reference one
or more of the FSL overview papers:

M.W. Woolrich, S. Jbabdi, B. Patenaude, M. Chappell, S. Makni, T. Behrens, C.
Beckmann, M. Jenkinson, S.M. Smith. Bayesian analysis of neuroimaging data in
FSL. NeuroImage, 45:S173-86, 2009

S.M. Smith, M. Jenkinson, M.W. Woolrich, C.F. Beckmann, T.E.J. Behrens, H.
Johansen-Berg, P.R. Bannister, M. De Luca, I. Drobnjak, D.E. Flitney, R. Niazy,
J. Saunders, J. Vickers, Y. Zhang, N. De Stefano, J.M. Brady, and P.M. Matthews.
Advances in functional and structural MR image analysis and implementation as
FSL. NeuroImage, 23(S1):208-19, 2004

M. Jenkinson, C.F. Beckmann, T.E. Behrens, M.W. Woolrich, S.M. Smith. FSL.
NeuroImage, 62:782-90, 2012.

URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
"""
# This file was auto generated by Styx.
# Do not edit this file directly.

from .aff2rigid import *
from .anatomical_average import *
from .apply_warp import *
from .applytopup import *
from .applywarp import *
from .applyxfm4_d import *
from .asl_file import *
from .asl_mfree import *
from .atlasquery import *
from .avscale import *
from .avw2fsl import *
from .b0calc import *
from .basil_var import *
from .baycest import *
from .bedpostx import *
from .bedpostx_datacheck import *
from .bedpostx_gpu import *
from .bedpostx_single_slice_sh import *
from .bet import *
from .bet2 import *
from .betsurf import *
from .bianca import *
from .bianca_cluster_stats import *
from .bianca_overlap_measures import *
from .bianca_perivent_deep import *
from .calc_grad_perc_dev import *
from .ccops import *
from .check_feat import *
from .cluster import *
from .cluster2html import *
from .concat_bvars import *
from .connectedcomp import *
from .convert_warp import *
from .convert_xfm import *
from .convertwarp import *
from .create_lut import *
from .cutoffcalc import *
from .design_ttest2 import *
from .distancemap import *
from .dtifit import *
from .dtigen import *
from .dual_regression import *
from .easythresh import *
from .eddy import *
from .eddy_combine import *
from .eddy_correct import *
from .eddy_quad import *
from .eddy_squad import *
from .epi_reg import *
from .estnoise import *
from .extracttxt import *
from .fabber import *
from .fabber_asl import *
from .fabber_cest import *
from .fabber_dce import *
from .fabber_dsc import *
from .fabber_dualecho import *
from .fabber_dwi import *
from .fabber_pet import *
from .fabber_qbold import *
from .fabber_t1 import *
from .fast import *
from .fdr import *
from .feat import *
from .feat_gm_prepare import *
from .feat_model import *
from .featquery import *
from .featregapply import *
from .film_cifti import *
from .film_gls import *
from .filmbabe import *
from .filmbabescript import *
from .find_the_biggest import *
from .first import *
from .first_flirt import *
from .first_mult_bcorr import *
from .first_roi_slicesdir import *
from .first_utils import *
from .flameo import *
from .flirt import *
from .flirt_average import *
from .fnirt import *
from .fnirtfileutils import *
from .fsl2ascii import *
from .fsl_anat import *
from .fsl_boxplot import *
from .fsl_deface import *
from .fsl_ents import *
from .fsl_fix_text import *
from .fsl_gen_3_d import *
from .fsl_get_standard import *
from .fsl_glm import *
from .fsl_histogram import *
from .fsl_motion_outliers import *
from .fsl_mrs import *
from .fsl_mrs_preproc import *
from .fsl_mrs_proc import *
from .fsl_mrs_sim import *
from .fsl_mrsi import *
from .fsl_mvlm import *
from .fsl_prepare_fieldmap import *
from .fsl_reg import *
from .fsl_regfilt import *
from .fsl_sbca import *
from .fsl_schurprod import *
from .fsl_sub import *
from .fsl_sub_config import *
from .fsl_sub_report import *
from .fsl_tsplot import *
from .fsladd import *
from .fslanimate import *
from .fslascii2img import *
from .fslcc import *
from .fslchfiletype import *
from .fslcomplex import *
from .fslcpgeom import *
from .fslcreatehd import *
from .fslfft import *
from .fslhd import *
from .fslinfo import *
from .fslinterleave import *
from .fslmaths import *
from .fslmeants import *
from .fslmerge import *
from .fslmodhd import *
from .fslnvols import *
from .fslorient import *
from .fslpspec import *
from .fslreorient2std import *
from .fslroi import *
from .fslselectvols import *
from .fslsize import *
from .fslslice import *
from .fslsmoothfill import *
from .fslsplit import *
from .fslstats import *
from .fslswapdim import *
from .fslswapdim_exe import *
from .fslval import *
from .fslvbm_1_bet import *
from .fslvbm_2_template import *
from .fslvbm_3_proc import *
from .ftoz import *
from .fugue import *
from .generate_brain import *
from .glm import *
from .gps import *
from .halfcosbasis import *
from .hist2prob import *
from .ica_aroma import *
from .image_stats import *
from .imcp import *
from .img2imgcoord import *
from .img2stdcoord import *
from .imglob import *
from .imln import *
from .immv import *
from .imrm import *
from .inv_warp import *
from .invfeatreg import *
from .invwarp import *
from .invwarp_exe import *
from .label2surf import *
from .lesion_filling import *
from .make_bianca_mask import *
from .make_dyadic_vectors import *
from .makerot import *
from .maskdyads import *
from .match_smoothing import *
from .maths import *
from .mccutup import *
from .mcflirt import *
from .mean import *
from .medianfilter import *
from .melodic import *
from .merge import *
from .merge_mrs_reports import *
from .midtrans import *
from .mist_display import *
from .mist_fa_reg import *
from .mm import *
from .morph_kernel import *
from .mp_diffpow_sh import *
from .mrsi_segment import *
from .msm import *
from .new_invwarp import *
from .old_betall import *
from .overlay import *
from .oxford_asl import *
from .pairreg import *
from .perfusion_subtract import *
from .pngappend import *
from .pnm_evs import *
from .pnm_stage1 import *
from .pointflirt import *
from .popp import *
from .possum import *
from .possum_interpmot_py import *
from .possum_matrix import *
from .possum_plot_py import *
from .possum_sum import *
from .prelude import *
from .prepare_fieldmap import *
from .prewhiten import *
from .probtrackx import *
from .proj_thresh import *
from .ptoz import *
from .pulse import *
from .pvmfit import *
from .qboot import *
from .randomise import *
from .rmsdiff import *
from .robust_fov import *
from .robustfov import *
from .roi import *
from .run_first import *
from .run_first_all import *
from .run_mesh_utils import *
from .siena import *
from .siena_cal import *
from .siena_diff import *
from .siena_flirt import *
from .siena_flow2std import *
from .sienax import *
from .sigloss import *
from .signal2image import *
from .slice import *
from .sliceanimate import *
from .slicer import *
from .slices import *
from .slices_summary import *
from .slicesdir import *
from .slicesmask import *
from .slicetimer import *
from .smooth_estimate import *
from .smoothest import *
from .spharm_rm import *
from .split import *
from .split_parts_gpu import *
from .standard_space_roi import *
from .std2imgcoord import *
from .surf2surf import *
from .surf_proj import *
from .surface_fdr import *
from .surfmaths import *
from .susan import *
from .svs_segment import *
from .swap_dimensions import *
from .swap_subjectwise import *
from .swap_voxelwise import *
from .swe import *
from .systemnoise import *
from .tbss_1_preproc import *
from .tbss_2_reg import *
from .tbss_3_postreg import *
from .tbss_4_prestats import *
from .tbss_deproject import *
from .tbss_fill import *
from .tbss_non_fa import *
from .tbss_skeleton import *
from .tbss_sym import *
from .tbss_x import *
from .tcalc import *
from .topup import *
from .tsplot import *
from .ttologp import *
from .ttoz import *
from .unconfound import *
from .vecreg import *
from .viena_quant import *
from .whirlgif import *
from .wpng import *
from .xfibres import *
from .xtract_blueprint import *
from .xtract_stats import *
from .xtract_viewer import *
from .zeropad import *
from .ztop import *



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_EXTRACT_GROUP_IN_CORR_METADATA = Metadata(
    id="b8b23d33ef8a1a71fb2a203bde388bdc12a7bbe9.boutiques",
    name="3dExtractGroupInCorr",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dExtractGroupInCorrOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_extract_group_in_corr(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dataset: OutputPathType
    """Output dataset reconstructed from GroupInCorr data"""


def v_3d_extract_group_in_corr(
    group_in_corr_file: InputPathType,
    runner: Runner | None = None,
) -> V3dExtractGroupInCorrOutputs:
    """
    This program breaks the collection of images from a GroupInCorr file back into
    individual AFNI 3D+time datasets.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dExtractGroupInCorr.html
    
    Args:
        group_in_corr_file: GroupInCorr file to extract datasets from (e.g.\
            AAA.grpincorr.niml).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dExtractGroupInCorrOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_EXTRACT_GROUP_IN_CORR_METADATA)
    cargs = []
    cargs.append("3dExtractGroupInCorr")
    cargs.append("[OPTIONS]")
    cargs.append(execution.input_file(group_in_corr_file))
    ret = V3dExtractGroupInCorrOutputs(
        root=execution.output_file("."),
        output_dataset=execution.output_file("[PREFIX]_[DATASET_LABEL].nii"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dExtractGroupInCorrOutputs",
    "V_3D_EXTRACT_GROUP_IN_CORR_METADATA",
    "v_3d_extract_group_in_corr",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__INSTALL_MEICA_DEMO_METADATA = Metadata(
    id="097156093c1a5e931817d9a774ed678044201473.boutiques",
    name="@Install_MEICA_Demo",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VInstallMeicaDemoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__install_meica_demo(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    readme_file: OutputPathType
    """Details of the demo archive"""


def v__install_meica_demo(
    demo_archive: InputPathType,
    runner: Runner | None = None,
) -> VInstallMeicaDemoOutputs:
    """
    Installs the demo archive for Prantik Kundu MEICA denoising tools. After the
    archive is downloaded and unpacked, see its README.txt for details.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@Install_MEICA_Demo.html
    
    Args:
        demo_archive: Demo archive to install (e.g. meica_demo.zip).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VInstallMeicaDemoOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__INSTALL_MEICA_DEMO_METADATA)
    cargs = []
    cargs.append("Install_MEICA_Demo")
    cargs.append(execution.input_file(demo_archive))
    ret = VInstallMeicaDemoOutputs(
        root=execution.output_file("."),
        readme_file=execution.output_file("README.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VInstallMeicaDemoOutputs",
    "V__INSTALL_MEICA_DEMO_METADATA",
    "v__install_meica_demo",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FSL_GLM_METADATA = Metadata(
    id="1c4d05f5c199ab578b87cdb2f914668e6d4d8a2e.boutiques",
    name="fsl_glm",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class FslGlmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fsl_glm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file_out: OutputPathType | None
    """Output file name for GLM parameter estimates (GLM betas)"""
    output_copes_out: OutputPathType | None
    """Output file name for COPEs (either as text file or image)"""
    output_zstats_out: OutputPathType | None
    """Output file name for Z-stats (either as text file or image)"""
    output_tstats_out: OutputPathType | None
    """Output file name for t-stats (either as text file or image)"""
    output_pvals_out: OutputPathType | None
    """Output file name for p-values of Z-stats (either as text file or
    image)"""
    output_fvals_out: OutputPathType | None
    """Output file name for F-value of full model fit"""
    output_pfvals_out: OutputPathType | None
    """Output file name for p-value for full model fit"""
    output_residuals_out: OutputPathType | None
    """Output file name for residuals"""
    output_varcb_out: OutputPathType | None
    """Output file name for variance of COPEs"""
    output_sigsq_out: OutputPathType | None
    """Output file name for residual noise variance sigma-square"""
    output_data_out: OutputPathType | None
    """Output file name for pre-processed data"""
    output_vnscales_out: OutputPathType | None
    """Output file name for scaling factors for variance normalisation"""


def fsl_glm(
    input_file: InputPathType,
    design_matrix: InputPathType,
    output_file: InputPathType | None = None,
    contrasts: InputPathType | None = None,
    mask_file: InputPathType | None = None,
    dof: float | None = None,
    design_norm_flag: bool = False,
    data_norm_flag: bool = False,
    vn_flag: bool = False,
    demean_flag: bool = False,
    output_copes: InputPathType | None = None,
    output_zstats: InputPathType | None = None,
    output_tstats: InputPathType | None = None,
    output_pvals: InputPathType | None = None,
    output_fvals: InputPathType | None = None,
    output_pfvals: InputPathType | None = None,
    output_residuals: InputPathType | None = None,
    output_varcb: InputPathType | None = None,
    output_sigsq: InputPathType | None = None,
    output_data: InputPathType | None = None,
    output_vnscales: InputPathType | None = None,
    vx_text: list[str] | None = None,
    vx_images: list[InputPathType] | None = None,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> FslGlmOutputs:
    """
    Simple GLM allowing temporal or spatial regression on either text data or
    images.
    
    Author: Christian F. Beckmann
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/GLM
    
    Args:
        input_file: Input file name (text matrix or 3D/4D image file).
        design_matrix: File name of the GLM design matrix (text time courses\
            for temporal regression or an image file for spatial regression).
        output_file: Output file name for GLM parameter estimates (GLM betas).
        contrasts: Matrix of t-statistics contrasts.
        mask_file: Mask image file name if input is image.
        dof: Set degrees-of-freedom explicitly.
        design_norm_flag: Switch on normalisation of the design matrix columns\
            to unit std. deviation.
        data_norm_flag: Switch on normalisation of the data time series to unit\
            std. deviation.
        vn_flag: Perform MELODIC variance-normalisation on data.
        demean_flag: Switch on de-meaning of design and data.
        output_copes: Output file name for COPEs (either as text file or image).
        output_zstats: Output file name for Z-stats (either as text file or\
            image).
        output_tstats: Output file name for t-stats (either as text file or\
            image).
        output_pvals: Output file name for p-values of Z-stats (either as text\
            file or image).
        output_fvals: Output file name for F-value of full model fit.
        output_pfvals: Output file name for p-value for full model fit.
        output_residuals: Output file name for residuals.
        output_varcb: Output file name for variance of COPEs.
        output_sigsq: Output file name for residual noise variance sigma-square.
        output_data: Output file name for pre-processed data.
        output_vnscales: Output file name for scaling factors for variance\
            normalisation.
        vx_text: List of text files containing text matrix confounds. Caution,\
            BETA option.
        vx_images: List of 4D images containing voxelwise confounds. Caution,\
            BETA option.
        help_flag: Display this help text.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FslGlmOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FSL_GLM_METADATA)
    cargs = []
    cargs.append("fsl_glm")
    cargs.append("-i")
    cargs.extend([
        "-i",
        execution.input_file(input_file)
    ])
    cargs.append("-d")
    cargs.extend([
        "-d",
        execution.input_file(design_matrix)
    ])
    cargs.append("-o")
    if output_file is not None:
        cargs.extend([
            "-o",
            execution.input_file(output_file)
        ])
    if contrasts is not None:
        cargs.extend([
            "-c",
            execution.input_file(contrasts)
        ])
    if mask_file is not None:
        cargs.extend([
            "-m",
            execution.input_file(mask_file)
        ])
    if dof is not None:
        cargs.extend([
            "--dof",
            str(dof)
        ])
    if design_norm_flag:
        cargs.append("--des_norm")
    if data_norm_flag:
        cargs.append("--dat_norm")
    if vn_flag:
        cargs.append("--vn")
    if demean_flag:
        cargs.append("--demean")
    if output_copes is not None:
        cargs.extend([
            "--out_cope",
            execution.input_file(output_copes)
        ])
    if output_zstats is not None:
        cargs.extend([
            "--out_z",
            execution.input_file(output_zstats)
        ])
    if output_tstats is not None:
        cargs.extend([
            "--out_t",
            execution.input_file(output_tstats)
        ])
    if output_pvals is not None:
        cargs.extend([
            "--out_p",
            execution.input_file(output_pvals)
        ])
    if output_fvals is not None:
        cargs.extend([
            "--out_f",
            execution.input_file(output_fvals)
        ])
    if output_pfvals is not None:
        cargs.extend([
            "--out_pf",
            execution.input_file(output_pfvals)
        ])
    if output_residuals is not None:
        cargs.extend([
            "--out_res",
            execution.input_file(output_residuals)
        ])
    if output_varcb is not None:
        cargs.extend([
            "--out_varcb",
            execution.input_file(output_varcb)
        ])
    if output_sigsq is not None:
        cargs.extend([
            "--out_sigsq",
            execution.input_file(output_sigsq)
        ])
    if output_data is not None:
        cargs.extend([
            "--out_data",
            execution.input_file(output_data)
        ])
    if output_vnscales is not None:
        cargs.extend([
            "--out_vnscales",
            execution.input_file(output_vnscales)
        ])
    if vx_text is not None:
        cargs.extend([
            "--vxt",
            *vx_text
        ])
    if vx_images is not None:
        cargs.extend([
            "--vxf",
            *[execution.input_file(f) for f in vx_images]
        ])
    if help_flag:
        cargs.append("-h")
    ret = FslGlmOutputs(
        root=execution.output_file("."),
        output_file_out=execution.output_file(pathlib.Path(output_file).name + ".nii.gz") if (output_file is not None) else None,
        output_copes_out=execution.output_file(pathlib.Path(output_copes).name + ".nii.gz") if (output_copes is not None) else None,
        output_zstats_out=execution.output_file(pathlib.Path(output_zstats).name + ".nii.gz") if (output_zstats is not None) else None,
        output_tstats_out=execution.output_file(pathlib.Path(output_tstats).name + ".nii.gz") if (output_tstats is not None) else None,
        output_pvals_out=execution.output_file(pathlib.Path(output_pvals).name + ".nii.gz") if (output_pvals is not None) else None,
        output_fvals_out=execution.output_file(pathlib.Path(output_fvals).name + ".nii.gz") if (output_fvals is not None) else None,
        output_pfvals_out=execution.output_file(pathlib.Path(output_pfvals).name + ".nii.gz") if (output_pfvals is not None) else None,
        output_residuals_out=execution.output_file(pathlib.Path(output_residuals).name + ".nii.gz") if (output_residuals is not None) else None,
        output_varcb_out=execution.output_file(pathlib.Path(output_varcb).name + ".nii.gz") if (output_varcb is not None) else None,
        output_sigsq_out=execution.output_file(pathlib.Path(output_sigsq).name + ".nii.gz") if (output_sigsq is not None) else None,
        output_data_out=execution.output_file(pathlib.Path(output_data).name + ".nii.gz") if (output_data is not None) else None,
        output_vnscales_out=execution.output_file(pathlib.Path(output_vnscales).name + ".nii.gz") if (output_vnscales is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FSL_GLM_METADATA",
    "FslGlmOutputs",
    "fsl_glm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SVS_SEGMENT_METADATA = Metadata(
    id="e55c5368e033ea1e2bc461214bfcbbcca79268de.boutiques",
    name="svs_segment",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class SvsSegmentOutputs(typing.NamedTuple):
    """
    Output object returned when calling `svs_segment(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    mask_file: OutputPathType | None
    """Generated mask file in T1 space"""
    segmentation_file: OutputPathType | None
    """Generated tissue segmentation file"""


def svs_segment(
    svs_file: InputPathType,
    t1_file: InputPathType | None = None,
    anat_dir: str | None = None,
    output_dir: str | None = None,
    filename_stem: str | None = None,
    mask_only_flag: bool = False,
    no_clean_flag: bool = False,
    runner: Runner | None = None,
) -> SvsSegmentOutputs:
    """
    FSL Magnetic Resonance Spectroscopy tool to construct mask in T1 space of an SVS
    voxel and generate a tissue segmentation file.
    
    Author: FSL Development Team
    
    Args:
        svs_file: SVS nifti file.
        t1_file: T1 nifti file.
        anat_dir: fsl_anat output directory.
        output_dir: Output directory.
        filename_stem: File name stem. _mask.nii.gz or _segmentation.json will\
            be added.
        mask_only_flag: Only perform masking stage, do not run fsl_anat if only\
            T1 passed.
        no_clean_flag: Don't clean intermediate output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SvsSegmentOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SVS_SEGMENT_METADATA)
    cargs = []
    cargs.append("svs_segment")
    cargs.append(execution.input_file(svs_file))
    if t1_file is not None:
        cargs.extend([
            "-t",
            execution.input_file(t1_file)
        ])
    if anat_dir is not None:
        cargs.extend([
            "-a",
            anat_dir
        ])
    if output_dir is not None:
        cargs.extend([
            "-o",
            output_dir
        ])
    if filename_stem is not None:
        cargs.extend([
            "-f",
            filename_stem
        ])
    if mask_only_flag:
        cargs.append("-m")
    if no_clean_flag:
        cargs.append("--no_clean")
    ret = SvsSegmentOutputs(
        root=execution.output_file("."),
        mask_file=execution.output_file(output_dir + "/" + filename_stem + "_mask.nii.gz") if (output_dir is not None and filename_stem is not None) else None,
        segmentation_file=execution.output_file(output_dir + "/" + filename_stem + "_segmentation.json") if (output_dir is not None and filename_stem is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SVS_SEGMENT_METADATA",
    "SvsSegmentOutputs",
    "svs_segment",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SLICETIMER_METADATA = Metadata(
    id="4720d9135dcde74cc0db05cf9e44c6a1d33e2f76.boutiques",
    name="slicetimer",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class SlicetimerOutputs(typing.NamedTuple):
    """
    Output object returned when calling `slicetimer(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_timeseries: OutputPathType
    """Output timeseries"""


def slicetimer(
    infile: InputPathType,
    runner: Runner | None = None,
) -> SlicetimerOutputs:
    """
    FMRIB's Interpolation for Slice Timing.
    
    Author: University of Oxford, FMRIB
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/Slicetimer
    
    Args:
        infile: Filename of input timeseries.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SlicetimerOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SLICETIMER_METADATA)
    cargs = []
    cargs.append("slicetimer")
    cargs.append("-i")
    cargs.extend([
        "-i",
        execution.input_file(infile)
    ])
    cargs.append("[-o")
    cargs.append("OUTPUT_FILE]")
    cargs.append("[--down]")
    cargs.append("[-r")
    cargs.append("TR_VALUE]")
    cargs.append("[-d")
    cargs.append("DIRECTION]")
    cargs.append("[--odd]")
    cargs.append("[--tcustom")
    cargs.append("TCUSTOM_FILE]")
    cargs.append("[--tglobal")
    cargs.append("TGLOBAL_VALUE]")
    cargs.append("[--ocustom")
    cargs.append("OCUSTOM_FILE]")
    cargs.append("[-v]")
    ret = SlicetimerOutputs(
        root=execution.output_file("."),
        output_timeseries=execution.output_file("[OUTPUT_FILE]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SLICETIMER_METADATA",
    "SlicetimerOutputs",
    "slicetimer",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FAT_PROC_CONVERT_DCM_DWIS_METADATA = Metadata(
    id="ee4b7dde4672e484d7c87cfda4720853b57ab16f.boutiques",
    name="fat_proc_convert_dcm_dwis",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class FatProcConvertDcmDwisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fat_proc_convert_dcm_dwis(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_nifti: OutputPathType
    """A NIFTI file with N volumes."""
    output_rvec: OutputPathType
    """A row-wise (3xN) bvec file of the (unit-magnitude) gradient
    orientations."""
    output_bval: OutputPathType
    """A row-wise (1xN) bval file of the gradient magnitudes."""
    output_mat_a: OutputPathType
    """A column-wise (Nx6) AFNI-style matrix file of the (scaled) b-matrix
    values."""
    output_mat_t: OutputPathType
    """A column-wise (Nx6) TORTOISE-style matrix file of the (scaled) b-matrix
    values."""
    output_cvec: OutputPathType
    """A column-wise (Nx3) bvec file of the (b-magn scaled) gradient
    orientations."""


def fat_proc_convert_dcm_dwis(
    dicom_dir: str,
    output_prefix: str,
    nifti_files: list[InputPathType] | None = None,
    bvec_files: list[InputPathType] | None = None,
    bval_files: list[InputPathType] | None = None,
    work_dir: str | None = None,
    orientation: str | None = None,
    flip_x: bool = False,
    flip_y: bool = False,
    flip_z: bool = False,
    no_flip: bool = False,
    qc_prefix: str | None = None,
    reorient_off: bool = False,
    no_clean: bool = False,
    no_cmd_out: bool = False,
    no_qc_view: bool = False,
    do_movie: str | None = None,
    runner: Runner | None = None,
) -> FatProcConvertDcmDwisOutputs:
    """
    Convert sets of DWIs in DICOM format into 'nicer' volume+grad format, reorient
    volumetric data, and glue together multiple sessions/directories of data.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/fat_proc_convert_dcm_dwis.html
    
    Args:
        dicom_dir: Directory of DICOM files of the DWI data with 'AP' phase\
            encoding. Can contain a wildcard expression for several directories.
        output_prefix: Prefix (and path) for output data (e.g., *.nii.gz,\
            *.bvec, *.bval files). Required.
        nifti_files: One or more NIFTI files of DWIs.
        bvec_files: One or more row-wise, gradient (unit-magnitude) files\
            (e.g., *.bvec).
        bval_files: One or more bvalue files (e.g., *.bval).
        work_dir: Optional working directory for intermediate files.
        orientation: Optional chance to reset orientation of the volume files\
            (default is currently 'RAI').
        flip_x: Flip gradients along the X-axis.
        flip_y: Flip gradients along the Y-axis.
        flip_z: Flip gradients along the Z-axis.
        no_flip: Prevent flipping of gradients (default).
        qc_prefix: Set the prefix for QC image files separately (default is\
            '').
        reorient_off: Turn off reorigin calculation and reorientation.
        no_clean: Do not remove the working directory of intermediate files\
            (default is to delete it).
        no_cmd_out: Do not save the command line call and location where it was\
            run.
        no_qc_view: Do not generate QC image files.
        do_movie: Generate a movie of the newly created dataset (AGIF or MPEG).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FatProcConvertDcmDwisOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FAT_PROC_CONVERT_DCM_DWIS_METADATA)
    cargs = []
    cargs.append("fat_proc_convert_dcm_dwis")
    cargs.append("-indir")
    cargs.append(dicom_dir)
    cargs.append("-prefix")
    cargs.append(output_prefix)
    cargs.append("-innii")
    if nifti_files is not None:
        cargs.extend([execution.input_file(f) for f in nifti_files])
    cargs.append("-inbvec")
    if bvec_files is not None:
        cargs.extend([execution.input_file(f) for f in bvec_files])
    cargs.append("-inbval")
    if bval_files is not None:
        cargs.extend([execution.input_file(f) for f in bval_files])
    cargs.append("-workdir")
    if work_dir is not None:
        cargs.append(work_dir)
    cargs.append("-orient")
    if orientation is not None:
        cargs.append(orientation)
    cargs.append("-origin_xyz")
    cargs.append("[ORIGIN_X]")
    cargs.append("[ORIGIN_Y]")
    cargs.append("[ORIGIN_Z]")
    cargs.append("-flip_x")
    if flip_x:
        cargs.append("-flip_x")
    cargs.append("-flip_y")
    if flip_y:
        cargs.append("-flip_y")
    cargs.append("-flip_z")
    if flip_z:
        cargs.append("-flip_z")
    cargs.append("-no_flip")
    if no_flip:
        cargs.append("-no_flip")
    cargs.append("-qc_prefix")
    if qc_prefix is not None:
        cargs.append(qc_prefix)
    cargs.append("-reorig_reorient_off")
    if reorient_off:
        cargs.append("-reorig_reorient_off")
    cargs.append("-no_clean")
    if no_clean:
        cargs.append("-no_clean")
    cargs.append("-no_cmd_out")
    if no_cmd_out:
        cargs.append("-no_cmd_out")
    cargs.append("-no_qc_view")
    if no_qc_view:
        cargs.append("-no_qc_view")
    cargs.append("-do_movie")
    if do_movie is not None:
        cargs.extend([
            "-do_movie",
            do_movie
        ])
    ret = FatProcConvertDcmDwisOutputs(
        root=execution.output_file("."),
        output_nifti=execution.output_file(output_prefix + ".nii.gz"),
        output_rvec=execution.output_file(output_prefix + ".rvec"),
        output_bval=execution.output_file(output_prefix + ".bval"),
        output_mat_a=execution.output_file(output_prefix + "_matA.dat"),
        output_mat_t=execution.output_file(output_prefix + "_matT.dat"),
        output_cvec=execution.output_file(output_prefix + "_cvec.dat"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FAT_PROC_CONVERT_DCM_DWIS_METADATA",
    "FatProcConvertDcmDwisOutputs",
    "fat_proc_convert_dcm_dwis",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_ALL_LABELS_TO_ROIS_METADATA = Metadata(
    id="2d16c8760004aef52593ed5e3781e8eefcf45fca.boutiques",
    name="cifti-all-labels-to-rois",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class CiftiAllLabelsToRoisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_all_labels_to_rois(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_all_labels_to_rois(
    label_in: InputPathType,
    map_: str,
    cifti_out: str,
    runner: Runner | None = None,
) -> CiftiAllLabelsToRoisOutputs:
    """
    Make rois from all labels in a cifti label map.
    
    The output cifti file is a dscalar file with a column (map) for each label
    in the specified input map, other than the ??? label, each of which contains
    a binary ROI of all brainordinates that are set to the corresponding label.
    
    Most of the time, specifying '1' for the <map> argument will do what is
    desired.
    
    Author: Washington University School of Medicin
    
    Args:
        label_in: the input cifti label file.
        map_: the number or name of the label map to use.
        cifti_out: the output cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiAllLabelsToRoisOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_ALL_LABELS_TO_ROIS_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-all-labels-to-rois")
    cargs.append(execution.input_file(label_in))
    cargs.append(map_)
    cargs.append(cifti_out)
    ret = CiftiAllLabelsToRoisOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_ALL_LABELS_TO_ROIS_METADATA",
    "CiftiAllLabelsToRoisOutputs",
    "cifti_all_labels_to_rois",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

INVWARP_METADATA = Metadata(
    id="778bad288efe828813a147cbd6f4c4be14fad8ca.boutiques",
    name="invwarp",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class InvwarpOutputs(typing.NamedTuple):
    """
    Output object returned when calling `invwarp(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_volume: OutputPathType
    """Output inverse warped volume"""


def invwarp(
    warpvol: InputPathType,
    outvol: str,
    refvol: InputPathType,
    relflag: bool = False,
    absflag: bool = False,
    noconstraintflag: bool = False,
    jmin: float | None = 0.01,
    jmax: float | None = 100.0,
    debugflag: bool = False,
    verboseflag: bool = False,
    runner: Runner | None = None,
) -> InvwarpOutputs:
    """
    Inverse warp tool from FSL suite.
    
    Author: University of Oxford (Jesper Andersson)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/FNIRT
    
    Args:
        warpvol: Filename for warp/shiftmap transform (volume).
        outvol: Filename for output (inverse warped) image.
        refvol: Filename for new reference image, i.e., what was originally the\
            input image (determines inverse warpvol's FOV and pixdims).
        relflag: Use relative warp convention: x' = x + w(x).
        absflag: Use absolute warp convention (default): x' = w(x).
        noconstraintflag: Do not apply the Jacobian constraint.
        jmin: Minimum acceptable Jacobian value for constraint (default 0.01).
        jmax: Maximum acceptable Jacobian value for constraint (default 100.0).
        debugflag: Turn on debugging output.
        verboseflag: Switch on diagnostic messages.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `InvwarpOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(INVWARP_METADATA)
    cargs = []
    cargs.append("invwarp")
    cargs.append("-w")
    cargs.extend([
        "-w",
        execution.input_file(warpvol)
    ])
    cargs.append("-o")
    cargs.extend([
        "-o",
        outvol
    ])
    cargs.append("-r")
    cargs.extend([
        "-r",
        execution.input_file(refvol)
    ])
    if relflag:
        cargs.append("--rel")
    if absflag:
        cargs.append("--abs")
    if noconstraintflag:
        cargs.append("--noconstraint")
    if jmin is not None:
        cargs.extend([
            "--jmin",
            str(jmin)
        ])
    if jmax is not None:
        cargs.extend([
            "--jmax",
            str(jmax)
        ])
    if debugflag:
        cargs.append("--debug")
    if verboseflag:
        cargs.append("-v")
    ret = InvwarpOutputs(
        root=execution.output_file("."),
        out_volume=execution.output_file(outvol),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "INVWARP_METADATA",
    "InvwarpOutputs",
    "invwarp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

POINTFLIRT_METADATA = Metadata(
    id="ee1b22cf597ec9ee03dbc9e66058c8eb17f83276.boutiques",
    name="pointflirt",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class PointflirtOutputs(typing.NamedTuple):
    """
    Output object returned when calling `pointflirt(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_matrix_file: OutputPathType | None
    """Affine matrix output file"""


def pointflirt(
    invol_coords: InputPathType,
    refvol_coords: InputPathType,
    out_matrix: InputPathType | None = None,
    use_vox: bool = False,
    vol_input: InputPathType | None = None,
    vol_ref: InputPathType | None = None,
    verbose_flag: bool = False,
    runner: Runner | None = None,
) -> PointflirtOutputs:
    """
    A tool to align point coordinates between volumes and compute affine
    transformation matrices.
    
    Author: Mark Jenkinson, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/
    
    Args:
        invol_coords: Filename of input volume coordinates.
        refvol_coords: Filename of reference volume coordinates.
        out_matrix: Filename of affine matrix output.
        use_vox: Use voxel coordinates, not mm, for input.
        vol_input: Filename of input volume (needed for --vox option).
        vol_ref: Filename of reference volume (needed for --vox option).
        verbose_flag: Switch on diagnostic messages.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `PointflirtOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(POINTFLIRT_METADATA)
    cargs = []
    cargs.append("pointflirt")
    cargs.append("-i")
    cargs.append(execution.input_file(invol_coords))
    cargs.append("-r")
    cargs.append(execution.input_file(refvol_coords))
    cargs.append("-o")
    if out_matrix is not None:
        cargs.extend([
            "-o",
            execution.input_file(out_matrix)
        ])
    if use_vox:
        cargs.append("--vox")
    if vol_input is not None:
        cargs.extend([
            "--invol",
            execution.input_file(vol_input)
        ])
    if vol_ref is not None:
        cargs.extend([
            "--refvol",
            execution.input_file(vol_ref)
        ])
    if verbose_flag:
        cargs.append("-v")
    ret = PointflirtOutputs(
        root=execution.output_file("."),
        output_matrix_file=execution.output_file(pathlib.Path(out_matrix).name) if (out_matrix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "POINTFLIRT_METADATA",
    "PointflirtOutputs",
    "pointflirt",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FIRST_METADATA = Metadata(
    id="5b6bb70b2b98ddae647d2c2cd52a91d828611c33.boutiques",
    name="first",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class FirstOutputs(typing.NamedTuple):
    """
    Output object returned when calling `first(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    segmented_output_image: OutputPathType
    """Segmented output image"""


def first(
    input_file: InputPathType,
    output_name: str,
    input_model: InputPathType,
    flirt_matrix: InputPathType,
    verbose: bool = False,
    help_: bool = False,
    input_model2: InputPathType | None = None,
    nmodes: float | None = None,
    intref: bool = False,
    multi_image_input: bool = False,
    binary_surface_output: bool = False,
    bmap_name: InputPathType | None = None,
    bvars: InputPathType | None = None,
    shcond: bool = False,
    loadbvars: bool = False,
    runner: Runner | None = None,
) -> FirstOutputs:
    """
    A command-line tool for segmenting subcortical structures in MRI images using
    models and transformations.
    
    Author: Brian Patenaude, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/FIRST
    
    Args:
        input_file: Filename of input image to be segmented.
        output_name: Output name.
        input_model: Filename of input model (the structure to be segmented).
        flirt_matrix: Filename of flirt matrix that transform input image to\
            MNI space (output of first_flirt).
        verbose: Switch on diagnostic messages.
        help_: Display help message.
        input_model2: Filename of second input model (the structure to be\
            segmented).
        nmodes: Specifies number of modes used.
        intref: Use structure specified by modelname2 as intensity reference.
        multi_image_input: Use structure specified by modelname2 as intensity\
            reference.
        binary_surface_output: Use structure specified by modelname2 as\
            intensity reference.
        bmap_name: Filename of conditional mapping matrix.
        bvars: Initialize using bvars from a previous segmentation. When using\
            with --shcond specifies the shape of the structure we are conditioning\
            on.
        shcond: Use conditional shape probability.
        loadbvars: Load initial parameter estimates from a previous\
            segmentation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FirstOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIRST_METADATA)
    cargs = []
    cargs.append("first")
    cargs.extend([
        "-i",
        execution.input_file(input_file)
    ])
    cargs.extend([
        "-k",
        output_name
    ])
    cargs.extend([
        "-m",
        execution.input_file(input_model)
    ])
    cargs.extend([
        "-l",
        execution.input_file(flirt_matrix)
    ])
    if verbose:
        cargs.append("-v")
    if help_:
        cargs.append("-h")
    if input_model2 is not None:
        cargs.extend([
            "-p",
            execution.input_file(input_model2)
        ])
    if nmodes is not None:
        cargs.extend([
            "-n",
            str(nmodes)
        ])
    if intref:
        cargs.append("--intref")
    if multi_image_input:
        cargs.append("--multiImageInput")
    if binary_surface_output:
        cargs.append("--binarySurfaceOutput")
    if bmap_name is not None:
        cargs.extend([
            "-b",
            execution.input_file(bmap_name)
        ])
    if bvars is not None:
        cargs.extend([
            "-o",
            execution.input_file(bvars)
        ])
    if shcond:
        cargs.append("--shcond")
    if loadbvars:
        cargs.append("--loadbvars")
    ret = FirstOutputs(
        root=execution.output_file("."),
        segmented_output_image=execution.output_file(output_name + "_seg.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FIRST_METADATA",
    "FirstOutputs",
    "first",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__XYZ_TO_IJK_METADATA = Metadata(
    id="014408a0c3580b9c28de243d9d3424b180fe652e.boutiques",
    name="@xyz_to_ijk",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VXyzToIjkOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__xyz_to_ijk(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType | None
    """Output file containing the translated indices"""


def v__xyz_to_ijk(
    inset: InputPathType,
    x_coord: float,
    y_coord: float,
    z_coord: float,
    prefix: str | None = None,
    runner: Runner | None = None,
) -> VXyzToIjkOutputs:
    """
    Helper script to convert (x, y, z) coordinates to (i, j, k) indices for a
    volumetric dataset.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/@xyz_to_ijk.html
    
    Args:
        inset: Volumetric file name (e.g. FILE.nii.gz).
        x_coord: Three coordinates (in units of the dataset, like mm).
        y_coord: Three coordinates (in units of the dataset, like mm).
        z_coord: Three coordinates (in units of the dataset, like mm).
        prefix: File name (including path) to output the three indices.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VXyzToIjkOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__XYZ_TO_IJK_METADATA)
    cargs = []
    cargs.append("@xyz_to_ijk")
    cargs.append("-inset")
    cargs.append(execution.input_file(inset))
    cargs.append("-xyz")
    cargs.append(str(x_coord))
    cargs.append(str(y_coord))
    cargs.append(str(z_coord))
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    ret = VXyzToIjkOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(prefix) if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VXyzToIjkOutputs",
    "V__XYZ_TO_IJK_METADATA",
    "v__xyz_to_ijk",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

INVFEATREG_METADATA = Metadata(
    id="ec6c695b85ee8f30eca8a86da04b682aec916c23.boutiques",
    name="invfeatreg",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class InvfeatregOutputs(typing.NamedTuple):
    """
    Output object returned when calling `invfeatreg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    inverted_warp_file: OutputPathType
    """Inverted warp file"""


def invfeatreg(
    warp_file: InputPathType,
    reference: InputPathType,
    output_file: str,
    verbose_flag: bool = False,
    runner: Runner | None = None,
) -> InvfeatregOutputs:
    """
    Inverse warp image using FNIRT.
    
    Author: Oxford Centre for Functional MRI of the Brain (FMRIB)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/FNIRT
    
    Args:
        warp_file: Input warp file (e.g., highres2standard_warp).
        reference: Reference image (e.g., highres.nii.gz).
        output_file: Output filename for the inverted warp file (e.g.,\
            standard2highres_warp).
        verbose_flag: Verbose (switch on diagnostic messages).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `InvfeatregOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(INVFEATREG_METADATA)
    cargs = []
    cargs.append("invwarp")
    cargs.append("-w")
    cargs.append(execution.input_file(warp_file))
    cargs.append("-r")
    cargs.append(execution.input_file(reference))
    cargs.append("-o")
    cargs.append(output_file)
    if verbose_flag:
        cargs.append("-v")
    ret = InvfeatregOutputs(
        root=execution.output_file("."),
        inverted_warp_file=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "INVFEATREG_METADATA",
    "InvfeatregOutputs",
    "invfeatreg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_1DPLOT_PY_METADATA = Metadata(
    id="ea514e31a969206c6fef70f023ec92ff968fc8c7.boutiques",
    name="1dplot.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V1dplotPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_1dplot_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image: OutputPathType
    """Output image file, default to .jpg"""


def v_1dplot_py(
    infiles: list[InputPathType],
    prefix: str,
    runner: Runner | None = None,
) -> V1dplotPyOutputs:
    """
    This program is for making images to visualize columns of numbers from 1D text
    files. It uses Python, particularly matplotlib, to create plots.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/1dplot.py.html
    
    Args:
        infiles: One or more file names of text files. Each column in this file\
            will be treated as a separate time series for plotting.
        prefix: Output filename or prefix. Default output image type is .jpg.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V1dplotPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_1DPLOT_PY_METADATA)
    cargs = []
    cargs.append("1dplot.py")
    cargs.extend([
        "-infiles",
        *[execution.input_file(f) for f in infiles]
    ])
    cargs.extend([
        "-prefix",
        prefix
    ])
    cargs.append("[OTHER_OPTIONS]")
    ret = V1dplotPyOutputs(
        root=execution.output_file("."),
        output_image=execution.output_file(prefix + ".jpg"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V1dplotPyOutputs",
    "V_1DPLOT_PY_METADATA",
    "v_1dplot_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FSL_SCHURPROD_METADATA = Metadata(
    id="b6df5ac8adcff7ee9b0b2fb96144e8303530b6c3.boutiques",
    name="fsl_schurprod",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class FslSchurprodOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fsl_schurprod(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_matrix_product: OutputPathType
    """Generated matrix product output file"""


def fsl_schurprod(
    input_file: InputPathType,
    design_file: InputPathType,
    output_file: str,
    regression_flag: bool = False,
    index: float | None = None,
    mask_file: InputPathType | None = None,
    verbose_flag: bool = False,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> FslSchurprodOutputs:
    """
    Generates element-wise matrix products or product of matrices against vectors
    from 4D data.
    
    Author: Christian F. Beckmann
    
    Args:
        input_file: Input file name (4D image file).
        design_file: ASCII text matrix of time series to be correlated.
        output_file: Output file base name.
        regression_flag: Use regression rather than correlation.
        index: Index of column in the design to be used for matrix product\
            calculation.
        mask_file: Mask image file name.
        verbose_flag: Switch on diagnostic messages.
        help_flag: Display this help text.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FslSchurprodOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FSL_SCHURPROD_METADATA)
    cargs = []
    cargs.append("fsl_schurprod")
    cargs.append("-i")
    cargs.append(execution.input_file(input_file))
    cargs.append("-d")
    cargs.append(execution.input_file(design_file))
    cargs.append("-o")
    cargs.append(output_file)
    if regression_flag:
        cargs.append("-r")
    if index is not None:
        cargs.extend([
            "-i",
            str(index)
        ])
    if mask_file is not None:
        cargs.extend([
            "-m",
            execution.input_file(mask_file)
        ])
    if verbose_flag:
        cargs.append("-v")
    if help_flag:
        cargs.append("-h")
    ret = FslSchurprodOutputs(
        root=execution.output_file("."),
        output_matrix_product=execution.output_file(output_file + ".nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FSL_SCHURPROD_METADATA",
    "FslSchurprodOutputs",
    "fsl_schurprod",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DAXIALIZE_METADATA = Metadata(
    id="4a31bcf01d5f3abdd670e6d35ef8f5e3b5aec53c.boutiques",
    name="3daxialize",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3daxializeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3daxialize(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType | None
    """Output dataset with axial slices orientation"""


def v_3daxialize(
    infile: InputPathType,
    prefix: str | None = None,
    verb: bool = False,
    sagittal: bool = False,
    coronal: bool = False,
    axial: bool = False,
    orient_code: str | None = None,
    frugal: bool = False,
    runner: Runner | None = None,
) -> V3daxializeOutputs:
    """
    Read and write dataset as new dataset with data brick oriented as axial slices.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3daxialize.html
    
    Args:
        infile: Dataset to be axially oriented.
        prefix: Use specified prefix for the new dataset. Default is\
            'axialize'.
        verb: Print out a progress report.
        sagittal: Write dataset in sagittal slice order.
        coronal: Write dataset in coronal slice order.
        axial: Write dataset in axial slice order, the default orientation.
        orient_code: Orientation code for output. 3 letters: one from {R,L},\
            {A,P}, {I,S}.
        frugal: Write data as it is rotated, saving memory. Not available with\
            NIFTI datasets.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3daxializeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DAXIALIZE_METADATA)
    cargs = []
    cargs.append("3daxialize")
    cargs.append("[OUTPUT_FILE]")
    cargs.append(execution.input_file(infile))
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if verb:
        cargs.append("-verb")
    if sagittal:
        cargs.append("-sagittal")
    if coronal:
        cargs.append("-coronal")
    if axial:
        cargs.append("-axial")
    if orient_code is not None:
        cargs.extend([
            "-orient",
            orient_code
        ])
    if frugal:
        cargs.append("-frugal")
    ret = V3daxializeOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(prefix + "+orig") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3daxializeOutputs",
    "V_3DAXIALIZE_METADATA",
    "v_3daxialize",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURFACE_DISTORTION_METADATA = Metadata(
    id="e97276a061c7333c75d0177ca285e8b2fa28ba0c.boutiques",
    name="surface-distortion",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class SurfaceDistortionSmooth:
    """
    smooth the area data.
    """
    sigma: float
    """the size of the smoothing kernel in mm, as sigma by default"""
    opt_fwhm: bool = False
    """kernel size is FWHM, not sigma"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-smooth")
        cargs.append(str(self.sigma))
        if self.opt_fwhm:
            cargs.append("-fwhm")
        return cargs


class SurfaceDistortionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surface_distortion(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output distortion metric"""


def surface_distortion(
    surface_reference: InputPathType,
    surface_distorted: InputPathType,
    metric_out: str,
    smooth: SurfaceDistortionSmooth | None = None,
    opt_caret5_method: bool = False,
    opt_edge_method: bool = False,
    opt_local_affine_method: bool = False,
    opt_log2: bool = False,
    runner: Runner | None = None,
) -> SurfaceDistortionOutputs:
    """
    Measure distortion between surfaces.
    
    This command, when not using -caret5-method, -edge-method, or
    -local-affine-method, is equivalent to using -surface-vertex-areas on each
    surface, smoothing both output metrics with the GEO_GAUSS_EQUAL method on
    the surface they came from if -smooth is specified, and then using the
    formula 'ln(distorted/reference)/ln(2)' on the smoothed results.
    
    When using -caret5-method, it uses the surface distortion method from
    caret5, which takes the base 2 log of the ratio of tile areas, then averages
    those results at each vertex, and then smooths the result on the reference
    surface.
    
    When using -edge-method, the -smooth option is ignored, and the output at
    each vertex is the average of 'abs(ln(refEdge/distortEdge)/ln(2))' over all
    edges connected to the vertex.
    
    When using -local-affine-method, the -smooth option is ignored. The output
    is two columns, the first is the area distortion ratio, and the second is
    anisotropic strain. These are calculated by an affine transform between
    matching triangles, and then averaged across the triangles of a vertex.
    
    Author: Washington University School of Medicin
    
    Args:
        surface_reference: the reference surface.
        surface_distorted: the distorted surface.
        metric_out: the output distortion metric.
        smooth: smooth the area data.
        opt_caret5_method: use the surface distortion method from caret5.
        opt_edge_method: calculate distortion of edge lengths rather than areas.
        opt_local_affine_method: calculate distortion by the local affines\
            between triangles.
        opt_log2: apply base-2 log transform.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceDistortionOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_DISTORTION_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-surface-distortion")
    cargs.append(execution.input_file(surface_reference))
    cargs.append(execution.input_file(surface_distorted))
    cargs.append(metric_out)
    if smooth is not None:
        cargs.extend(smooth.run(execution))
    if opt_caret5_method:
        cargs.append("-caret5-method")
    if opt_edge_method:
        cargs.append("-edge-method")
    if opt_local_affine_method:
        cargs.append("-local-affine-method")
    if opt_log2:
        cargs.append("-log2")
    ret = SurfaceDistortionOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(metric_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURFACE_DISTORTION_METADATA",
    "SurfaceDistortionOutputs",
    "SurfaceDistortionSmooth",
    "surface_distortion",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

VOLUME_MERGE_METADATA = Metadata(
    id="511518c9b9fab21f202bab4c023c973275bfe85b.boutiques",
    name="volume-merge",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class VolumeMergeUpTo:
    """
    use an inclusive range of subvolumes.
    """
    last_subvol: str
    """the number or name of the last subvolume to include"""
    opt_reverse: bool = False
    """use the range in reverse order"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-up-to")
        cargs.append(self.last_subvol)
        if self.opt_reverse:
            cargs.append("-reverse")
        return cargs


@dataclasses.dataclass
class VolumeMergeSubvolume:
    """
    select a single subvolume to use.
    """
    subvol: str
    """the subvolume number or name"""
    up_to: VolumeMergeUpTo | None = None
    """use an inclusive range of subvolumes"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-subvolume")
        cargs.append(self.subvol)
        if self.up_to is not None:
            cargs.extend(self.up_to.run(execution))
        return cargs


@dataclasses.dataclass
class VolumeMergeVolume:
    """
    specify an input volume file.
    """
    volume_in: InputPathType
    """a volume file to use subvolumes from"""
    subvolume: list[VolumeMergeSubvolume] | None = None
    """select a single subvolume to use"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-volume")
        cargs.append(execution.input_file(self.volume_in))
        if self.subvolume is not None:
            cargs.extend([a for c in [s.run(execution) for s in self.subvolume] for a in c])
        return cargs


class VolumeMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `volume_merge(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def volume_merge(
    volume_out: str,
    volume: list[VolumeMergeVolume] | None = None,
    runner: Runner | None = None,
) -> VolumeMergeOutputs:
    """
    Merge volume files into a new file.
    
    Takes one or more volume files and constructs a new volume file by
    concatenating subvolumes from them. The input volume files must have the
    same volume space.
    
    Example: wb_command -volume-merge out.nii -volume first.nii -subvolume 1
    -volume second.nii
    
    This example would take the first subvolume from first.nii, followed by all
    subvolumes from second.nii, and write these to out.nii.
    
    Author: Washington University School of Medicin
    
    Args:
        volume_out: the output volume file.
        volume: specify an input volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeMergeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_MERGE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-merge")
    cargs.append(volume_out)
    if volume is not None:
        cargs.extend([a for c in [s.run(execution) for s in volume] for a in c])
    ret = VolumeMergeOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(volume_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VOLUME_MERGE_METADATA",
    "VolumeMergeOutputs",
    "VolumeMergeSubvolume",
    "VolumeMergeUpTo",
    "VolumeMergeVolume",
    "volume_merge",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_REMLFIT_METADATA = Metadata(
    id="9fb81de20c23d0c04b8b45344a5e91f20b4ba1b5.boutiques",
    name="3dREMLfit",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dRemlfitOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_remlfit(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType | None
    """Main default output of 3dREMLfit"""
    rvar_file: OutputPathType | None
    """REML variance parameters"""
    rbeta_file: OutputPathType | None
    """REML beta weights"""
    rbuck_file: OutputPathType | None
    """REML estimates and statistics"""
    rfitts_file: OutputPathType | None
    """REML fitted model"""
    rerrts_file: OutputPathType | None
    """REML residuals"""


def v_3d_remlfit(
    input_file: InputPathType,
    regression_matrix: InputPathType,
    baseline_files: list[str] | None = None,
    sort_nods: bool = False,
    temp_storage: bool = False,
    mask: InputPathType | None = None,
    output_prefix: str | None = None,
    go_for_it: bool = False,
    max_b_param: float | None = None,
    grid_param: float | None = None,
    negative_corr: bool = False,
    runner: Runner | None = None,
) -> V3dRemlfitOutputs:
    """
    Generalized least squares time series fit, with REML estimation of the temporal
    auto-correlation structure.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dREMLfit.html
    
    Args:
        input_file: Read time series dataset.
        regression_matrix: Read the regression matrix, which should have been\
            output from 3dDeconvolve via the '-x1D' option.
        baseline_files: Add baseline model columns to the matrix. Each column\
            in the specified .1D file will be appended to the matrix.
        sort_nods: If '-dsort' is used, the output datasets reflect the impact\
            of the voxel-wise regressor(s). If you want to compare those results to\
            the case where you did NOT give the '-dsort' option, then also use\
            '-dsort_nods'.
        temp_storage: Write intermediate output to disk, to economize on RAM.
        mask: Read dataset as a mask for the input; voxels outside the mask\
            will not be fit by the regression model.
        output_prefix: Dataset prefix for saving REML variance parameters.
        go_for_it: Force the program to continue past a failed collinearity\
            check.
        max_b_param: Set max allowed MA b parameter.
        grid_param: Set the number of grid divisions in the (a,b) grid.
        negative_corr: Allows negative correlations to be used.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dRemlfitOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_REMLFIT_METADATA)
    cargs = []
    cargs.append("3dREMLfit")
    cargs.extend([
        "-input",
        execution.input_file(input_file)
    ])
    cargs.extend([
        "-matrix",
        execution.input_file(regression_matrix)
    ])
    if baseline_files is not None:
        cargs.extend([
            "-addbase",
            *baseline_files
        ])
    if sort_nods:
        cargs.append("-dsort_nods")
    if temp_storage:
        cargs.append("-usetemp")
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if output_prefix is not None:
        cargs.extend([
            "-Rvar",
            output_prefix
        ])
    if go_for_it:
        cargs.append("-GOFORIT")
    if max_b_param is not None:
        cargs.extend([
            "-MAXb",
            str(max_b_param)
        ])
    if grid_param is not None:
        cargs.extend([
            "-Grid",
            str(grid_param)
        ])
    if negative_corr:
        cargs.append("-NEGcor")
    ret = V3dRemlfitOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(output_prefix + ".nii.gz") if (output_prefix is not None) else None,
        rvar_file=execution.output_file(output_prefix + "_Rvar.nii.gz") if (output_prefix is not None) else None,
        rbeta_file=execution.output_file(output_prefix + "_Rbeta.nii.gz") if (output_prefix is not None) else None,
        rbuck_file=execution.output_file(output_prefix + "_Rbuck.nii.gz") if (output_prefix is not None) else None,
        rfitts_file=execution.output_file(output_prefix + "_Rfitts.nii.gz") if (output_prefix is not None) else None,
        rerrts_file=execution.output_file(output_prefix + "_Rerrts.nii.gz") if (output_prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dRemlfitOutputs",
    "V_3D_REMLFIT_METADATA",
    "v_3d_remlfit",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

METRIC_TO_VOLUME_MAPPING_METADATA = Metadata(
    id="9d8f8f073cc183066cda45f63e4572dc70180d1b.boutiques",
    name="metric-to-volume-mapping",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class MetricToVolumeMappingRibbonConstrained:
    """
    use ribbon constrained mapping algorithm.
    """
    inner_surf: InputPathType
    """the inner surface of the ribbon"""
    outer_surf: InputPathType
    """the outer surface of the ribbon"""
    opt_voxel_subdiv_subdiv_num: int | None = None
    """voxel divisions while estimating voxel weights: number of subdivisions,
    default 3"""
    opt_greedy: bool = False
    """instead of antialiasing partial-volumed voxels, put full metric values
    (legacy behavior)"""
    opt_thick_columns: bool = False
    """use overlapping columns (legacy method)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-ribbon-constrained")
        cargs.append(execution.input_file(self.inner_surf))
        cargs.append(execution.input_file(self.outer_surf))
        if self.opt_voxel_subdiv_subdiv_num is not None:
            cargs.extend([
                "-voxel-subdiv",
                str(self.opt_voxel_subdiv_subdiv_num)
            ])
        if self.opt_greedy:
            cargs.append("-greedy")
        if self.opt_thick_columns:
            cargs.append("-thick-columns")
        return cargs


class MetricToVolumeMappingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `metric_to_volume_mapping(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def metric_to_volume_mapping(
    metric: InputPathType,
    surface: InputPathType,
    volume_space: InputPathType,
    volume_out: str,
    opt_nearest_vertex_distance: float | None = None,
    ribbon_constrained: MetricToVolumeMappingRibbonConstrained | None = None,
    runner: Runner | None = None,
) -> MetricToVolumeMappingOutputs:
    """
    Map metric file to volume.
    
    Maps values from a metric file into a volume file. You must specify exactly
    one mapping method option. The -nearest-vertex method uses the value from
    the vertex closest to the voxel center (useful for integer values). The
    -ribbon-constrained method uses the same method as in
    -volume-to-surface-mapping, then uses the weights in reverse. Mapping to
    lower resolutions than the mesh may require a larger -voxel-subdiv value in
    order to have all of the surface data participate.
    
    Author: Washington University School of Medicin
    
    Args:
        metric: the input metric file.
        surface: the surface to use coordinates from.
        volume_space: a volume file in the desired output volume space.
        volume_out: the output volume file.
        opt_nearest_vertex_distance: use the value from the vertex closest to\
            the voxel center: how far from the surface to map values to voxels, in\
            mm.
        ribbon_constrained: use ribbon constrained mapping algorithm.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricToVolumeMappingOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_TO_VOLUME_MAPPING_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-to-volume-mapping")
    cargs.append(execution.input_file(metric))
    cargs.append(execution.input_file(surface))
    cargs.append(execution.input_file(volume_space))
    cargs.append(volume_out)
    if opt_nearest_vertex_distance is not None:
        cargs.extend([
            "-nearest-vertex",
            str(opt_nearest_vertex_distance)
        ])
    if ribbon_constrained is not None:
        cargs.extend(ribbon_constrained.run(execution))
    ret = MetricToVolumeMappingOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(volume_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "METRIC_TO_VOLUME_MAPPING_METADATA",
    "MetricToVolumeMappingOutputs",
    "MetricToVolumeMappingRibbonConstrained",
    "metric_to_volume_mapping",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SHBASIS_METADATA = Metadata(
    id="707c36ba783241cb02f7ec460fd290130572ea03.boutiques",
    name="shbasis",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class ShbasisConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class ShbasisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `shbasis(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def shbasis(
    sh: list[InputPathType],
    convert: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[ShbasisConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> ShbasisOutputs:
    """
    Examine the values in spherical harmonic images to estimate (and optionally
    change) the SH basis used.
    
    In previous versions of MRtrix, the convention used for storing spherical
    harmonic coefficients was a non-orthonormal basis (the m!=0 coefficients
    were a factor of sqrt(2) too large). This error has been rectified in newer
    versions of MRtrix, but will cause issues if processing SH data that was
    generated using an older version of MRtrix (or vice-versa).
    
    This command provides a mechanism for testing the basis used in storage of
    image data representing a spherical harmonic series per voxel, and allows
    the user to forcibly modify the raw image data to conform to the desired
    basis.
    
    Note that the "force_*" conversion choices should only be used in cases
    where this command has previously been unable to automatically determine the
    SH basis from the image data, but the user themselves are confident of the
    SH basis of the data.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: Robert E. Smith (robert.smith@florey.edu.au)
    
    URL: https://mrtrix.readthedocs.io/en/latest/reference/commands/shbasis.html
    
    Args:
        sh: the input image(s) of SH coefficients.
        convert: convert the image data in-place to the desired basis; options\
            are: old,new,force_oldtonew,force_newtoold.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ShbasisOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SHBASIS_METADATA)
    cargs = []
    cargs.append("shbasis")
    if convert is not None:
        cargs.extend([
            "-convert",
            convert
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.extend([execution.input_file(f) for f in sh])
    ret = ShbasisOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SHBASIS_METADATA",
    "ShbasisConfig",
    "ShbasisOutputs",
    "shbasis",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TFIM_METADATA = Metadata(
    id="e2647e37a1429074565987a0998548003c59d175.boutiques",
    name="tfim",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class TfimOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tfim(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    diff_output: OutputPathType
    """Difference image output. Default prefix is 'tfim'."""
    tspm_output: OutputPathType
    """T-statistic of difference. Default prefix is 'tfim'."""
    corr_output: OutputPathType
    """Equivalent correlation statistic output. Written if -eqcorr is used."""


def tfim(
    set1_images: list[InputPathType],
    set2_images: list[InputPathType],
    base1_value: float | None = None,
    runner: Runner | None = None,
) -> TfimOutputs:
    """
    MCW TFIM: t-tests on sets of functional images.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/tfim.html
    
    Args:
        set1_images: First set of image files.
        set2_images: Second set of image files.
        base1_value: Base value for the first set of images. Used for Usage 2.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TfimOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TFIM_METADATA)
    cargs = []
    cargs.append("tfim")
    cargs.append("[OPTIONS]")
    cargs.append("-set1")
    cargs.extend([
        "-set1",
        *[execution.input_file(f) for f in set1_images]
    ])
    cargs.append("-set2")
    cargs.extend([
        "-set2",
        *[execution.input_file(f) for f in set2_images]
    ])
    cargs.append("-base1")
    if base1_value is not None:
        cargs.extend([
            "-base1",
            str(base1_value)
        ])
    ret = TfimOutputs(
        root=execution.output_file("."),
        diff_output=execution.output_file("[PREFIX].diff"),
        tspm_output=execution.output_file("[PREFIX].tspm"),
        corr_output=execution.output_file("[PREFIX].corr"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TFIM_METADATA",
    "TfimOutputs",
    "tfim",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ADJUNCT_APQC_TSNR_GENERAL_METADATA = Metadata(
    id="48c52de6decf19ca4c3c5da02857ce697534bc6d.boutiques",
    name="adjunct_apqc_tsnr_general",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class AdjunctApqcTsnrGeneralOutputs(typing.NamedTuple):
    """
    Output object returned when calling `adjunct_apqc_tsnr_general(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def adjunct_apqc_tsnr_general(
    runner: Runner | None = None,
) -> AdjunctApqcTsnrGeneralOutputs:
    """
    An adjunct program for making TSNR plots for APQC.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/adjunct_apqc_tsnr_general.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AdjunctApqcTsnrGeneralOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ADJUNCT_APQC_TSNR_GENERAL_METADATA)
    cargs = []
    cargs.append("adjunct_apqc_tsnr_general")
    cargs.append("[OPTIONS]")
    ret = AdjunctApqcTsnrGeneralOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ADJUNCT_APQC_TSNR_GENERAL_METADATA",
    "AdjunctApqcTsnrGeneralOutputs",
    "adjunct_apqc_tsnr_general",
]
